@@ -6,30 +6,15 @@ has tools for background estimation, ePSF building, PSF matching,
 centroiding, and morphological measurements.
 """
 
+import os
+
+
 # Affiliated packages may add whatever they like to this file, but
 # should keep this content at the top.
 # ----------------------------------------------------------------------------
-from ._astropy_init import *       # noqa
+from ._astropy_init import *  # noqa
 # ----------------------------------------------------------------------------
 
-# Enforce Python version check during package import.
-# This is the same check as the one at the top of setup.py
-import sys
-
-__minimum_python_version__ = '3.5'
-__minimum_numpy_version__ = '1.11'
-
-
-class UnsupportedPythonError(Exception):
-    pass
-
-
-if (sys.version_info <
-        tuple((int(val) for val in __minimum_python_version__.split('.')))):
-    raise UnsupportedPythonError('Photutils does not support Python < {}'
-                                 .format(__minimum_python_version__))
-
-
 if not _ASTROPY_SETUP_:  # noqa
     from .aperture import *  # noqa
     from .background import *  # noqa
@@ -38,3 +23,17 @@ if not _ASTROPY_SETUP_:  # noqa
     from .morphology import *  # noqa
     from .psf import *  # noqa
     from .segmentation import *  # noqa
+
+
+# Set the bibtex entry to the article referenced in CITATION.
+def _get_bibtex():
+    citation_file = os.path.join(os.path.dirname(__file__), 'CITATION')
+
+    with open(citation_file, 'r') as citation:
+        refs = citation.read().split('@misc')[1:]
+        if len(refs) == 0: return ''
+        bibtexreference = "@misc{0}".format(refs[0])
+    return bibtexreference
+
+
+__citation__ = __bibtex__ = _get_bibtex()
