@@ -66,10 +66,11 @@ class RectangularMaskMixin:
 
         Returns
         -------
-        mask : `~photutils.ApertureMask` or list of `~photutils.ApertureMask`
+        mask : `~photutils.aperture.ApertureMask` or list of `~photutils.aperture.ApertureMask`
             A mask for the aperture.  If the aperture is scalar then a
-            single `~photutils.ApertureMask` is returned, otherwise a
-            list of `~photutils.ApertureMask` is returned.
+            single `~photutils.aperture.ApertureMask` is returned,
+            otherwise a list of `~photutils.aperture.ApertureMask` is
+            returned.
         """
 
         _, subpixels = self._translate_mask_mode(method, subpixels,
