@@ -1,29 +1,22 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 """
-Functions for centroiding sources and measuring their morphological
-properties.
+The module contains tools for centroiding sources.
 """
 
 import inspect
-import sys
 import warnings
 
-import numpy as np
 from astropy.modeling import Fittable2DModel, Parameter
-from astropy.modeling.models import (Gaussian1D, Gaussian2D, Const1D,
-                                     Const2D, CONSTRAINTS_DOC)
 from astropy.modeling.fitting import LevMarLSQFitter
+from astropy.modeling.models import (CONSTRAINTS_DOC, Const1D, Const2D,
+                                     Gaussian1D, Gaussian2D)
 from astropy.nddata.utils import overlap_slices
 from astropy.utils.exceptions import AstropyUserWarning
-
+import numpy as np
 
 __all__ = ['GaussianConst2D', 'centroid_com', 'gaussian1d_moments',
            'fit_2dgaussian', 'centroid_1dg', 'centroid_2dg',
-           'centroid_sources']
-
-
-class _GaussianConst1D(Const1D + Gaussian1D):
-    """A model for a 1D Gaussian plus a constant."""
+           'centroid_sources', 'centroid_epsf']
 
 
 class GaussianConst2D(Fittable2DModel):
@@ -34,20 +27,26 @@ class GaussianConst2D(Fittable2DModel):
     ----------
     constant : float
         Value of the constant.
+
     amplitude : float
         Amplitude of the Gaussian.
+
     x_mean : float
         Mean of the Gaussian in x.
+
     y_mean : float
         Mean of the Gaussian in y.
+
     x_stddev : float
         Standard deviation of the Gaussian in x.
         ``x_stddev`` and ``y_stddev`` must be specified unless a covariance
         matrix (``cov_matrix``) is input.
+
     y_stddev : float
         Standard deviation of the Gaussian in y.
         ``x_stddev`` and ``y_stddev`` must be specified unless a covariance
         matrix (``cov_matrix``) is input.
+
     theta : float, optional
         Rotation angle in radians. The rotation angle increases
         counterclockwise.
@@ -75,7 +74,7 @@ class GaussianConst2D(Fittable2DModel):
 GaussianConst2D.__doc__ += CONSTRAINTS_DOC
 
 
-def centroid_com(data, mask=None):
+def centroid_com(data, mask=None, oversampling=1.):
     """
     Calculate the centroid of an n-dimensional array as its "center of
     mass" determined from moments.
@@ -92,6 +91,11 @@ def centroid_com(data, mask=None):
         A boolean mask, with the same shape as ``data``, where a `True`
         value indicates the corresponding element of ``data`` is masked.
 
+    oversampling : float or tuple of two floats, optional
+        Oversampling factors of pixel indices. If ``oversampling`` is a scalar
+        this is treated as both x and y directions having the same oversampling
+        factor; otherwise it is treated as ``(x_oversamp, y_oversamp)``.
+
     Returns
     -------
     centroid : `~numpy.ndarray`
@@ -99,7 +103,14 @@ def centroid_com(data, mask=None):
         or ``(x, y, z)``), not numpy axis order.
     """
 
-    data = data.astype(np.float)
+    oversampling = np.atleast_1d(oversampling)
+    if len(oversampling) == 1:
+        oversampling = np.repeat(oversampling, 2)
+    # as these need to match data we reverse the order to (y, x)
+    oversampling = oversampling[::-1]
+    if np.any(oversampling <= 0):
+        raise ValueError('Oversampling factors must all be positive numbers.')
+    data = data.astype(float)
 
     if mask is not None and mask is not np.ma.nomask:
         mask = np.asarray(mask, dtype=bool)
@@ -117,7 +128,7 @@ def centroid_com(data, mask=None):
     indices = np.ogrid[[slice(0, i) for i in data.shape]]
 
     # note the output array is reversed to give (x, y) order
-    return np.array([np.sum(indices[axis] * data) / total
+    return np.array([np.sum(indices[axis] * data) / total / oversampling[axis]
                      for axis in range(data.ndim)])[::-1]
 
 
@@ -236,7 +247,7 @@ def fit_2dgaussian(data, error=None, mask=None):
     # values can yield undefined Gaussian parameters, e.g. x/y_stddev).
     props = data_properties(data - np.min(data), mask=mask)
 
-    init_const = 0.    # subtracted data minimum above
+    init_const = 0.  # subtracted data minimum above
     init_amplitude = np.ptp(data)
     g_init = GaussianConst2D(constant=init_const, amplitude=init_amplitude,
                              x_mean=props.xcentroid.value,
@@ -299,25 +310,24 @@ def centroid_1dg(data, error=None, mask=None):
         data.mask |= error.mask
 
         error.mask = data.mask
-        xy_error = np.array([np.sqrt(np.ma.sum(error**2, axis=i))
-                             for i in [0, 1]])
+        xy_error = [np.sqrt(np.ma.sum(error**2, axis=i)) for i in [0, 1]]
         xy_weights = [(1.0 / xy_error[i].clip(min=1.e-30)) for i in [0, 1]]
     else:
         xy_weights = [np.ones(data.shape[i]) for i in [1, 0]]
 
-    # assign zero weight to masked pixels
-    if data.mask is not np.ma.nomask:
+    # assign zero weight where an entire row or column is masked
+    if np.any(data.mask):
         bad_idx = [np.all(data.mask, axis=i) for i in [0, 1]]
         for i in [0, 1]:
             xy_weights[i][bad_idx[i]] = 0.
 
-    xy_data = np.array([np.ma.sum(data, axis=i) for i in [0, 1]])
+    xy_data = [np.ma.sum(data, axis=i).data for i in [0, 1]]
 
     constant_init = np.ma.min(data)
     centroid = []
     for (data_i, weights_i) in zip(xy_data, xy_weights):
         params_init = gaussian1d_moments(data_i)
-        g_init = _GaussianConst1D(constant_init, *params_init)
+        g_init = Const1D(constant_init) + Gaussian1D(*params_init)
         fitter = LevMarLSQFitter()
         x = np.arange(data_i.size)
         g_fit = fitter(g_init, x, data_i, weights=weights_i)
@@ -410,7 +420,7 @@ def centroid_sources(data, xpos, ypos, box_size=11, footprint=None,
         calculate the centroid of a 2D array.  The ``centroid_func``
         must accept a 2D `~numpy.ndarray`, have a ``mask`` keyword and
         optionally an ``error`` keyword.  The callable object must
-        return a tuple of two 1D `~numpy.ndarray`\s, representing the x
+        return a tuple of two 1D `~numpy.ndarray`\\s, representing the x
         and y centroids.  The default is
         `~photutils.centroids.centroid_com`.
 
@@ -430,12 +440,12 @@ def centroid_sources(data, xpos, ypos, box_size=11, footprint=None,
     if footprint is None:
         if box_size is None:
             raise ValueError('box_size or footprint must be defined.')
-        else:
-            box_size = np.atleast_1d(box_size)
-            if len(box_size) == 1:
-                box_size = np.repeat(box_size, 2)
-            if len(box_size) != 2:
-                raise ValueError('box_size must have 1 or 2 elements.')
+
+        box_size = np.atleast_1d(box_size)
+        if len(box_size) == 1:
+            box_size = np.repeat(box_size, 2)
+        if len(box_size) != 2:
+            raise ValueError('box_size must have 1 or 2 elements.')
 
         footprint = np.ones(box_size, dtype=bool)
     else:
@@ -483,3 +493,109 @@ def centroid_sources(data, xpos, ypos, box_size=11, footprint=None,
         ycentroids.append(ycen + slices_large[0].start)
 
     return np.array(xcentroids), np.array(ycentroids)
+
+
+def centroid_epsf(data, mask=None, oversampling=4, shift_val=0.5):
+    """
+    Calculates centering shift of data using pixel symmetry, as
+    described by Anderson and King (2000; PASP 112, 1360) in their
+    ePSF-fitting algorithm.
+
+    Calculate the shift of a 2-dimensional symmetric image based on the
+    asymmetry between f(x, N) and f(x, -N), along with the differential
+    df/dy(x, shift_val) and df/dy(x, -shift_val). Invalid values (e.g.
+    NaNs or infs) in the ``data`` array are automatically masked.
+
+    Parameters
+    ----------
+    data : array_like
+        The input n-dimensional array.
+    mask : array_like (bool), optional
+        A boolean mask, with the same shape as ``data``, where a `True`
+        value indicates the corresponding element of ``data`` is masked.
+    oversampling : float or tuple of two floats, optional
+        Oversampling factors of pixel indices. If ``oversampling`` is a
+        scalar this is treated as both x and y directions having the
+        same oversampling factor.  Otherwise it is treated as
+        ``(x_oversamp, y_oversamp)``.
+    shift_val : float, optional
+        The undersampled value at which to compute the shifts. Default
+        is half a pixel. If supplied, must be a strictly positive
+        number.
+
+    Returns
+    -------
+    centroid : tuple of floats
+        The (x, y) coordinates of the centroid in pixel order.
+    """
+
+    oversampling = np.atleast_1d(oversampling)
+    if len(oversampling) == 1:
+        oversampling = np.repeat(oversampling, 2)
+    if np.any(oversampling <= 0):
+        raise ValueError('Oversampling factors must all be positive numbers.')
+
+    data = data.astype(float)
+
+    if mask is not None and mask is not np.ma.nomask:
+        mask = np.asarray(mask, dtype=bool)
+        if data.shape != mask.shape:
+            raise ValueError('data and mask must have the same shape.')
+        data[mask] = 0.
+
+    if shift_val <= 0:
+        raise ValueError('shift_val must be a positive number.')
+
+    # Assume the center of the ePSF is the middle of an odd-sized grid.
+    xidx_0 = int((data.shape[1] - 1) / 2)
+    x_0 = np.arange(data.shape[1], dtype=float)[xidx_0] / oversampling[0]
+    yidx_0 = int((data.shape[0] - 1) / 2)
+    y_0 = np.arange(data.shape[0], dtype=float)[yidx_0] / oversampling[1]
+
+    x_shiftidx = np.around((shift_val * oversampling[0])).astype(int)
+    y_shiftidx = np.around((shift_val * oversampling[1])).astype(int)
+
+    badidx = ~np.isfinite([data[y, x]
+                           for x in [xidx_0, xidx_0+x_shiftidx,
+                                     xidx_0+x_shiftidx-1, xidx_0+x_shiftidx+1]
+                           for y in [yidx_0, yidx_0+y_shiftidx,
+                                     yidx_0+y_shiftidx-1,
+                                     yidx_0+y_shiftidx+1]])
+    if np.any(badidx):
+        raise ValueError('One or more centroiding pixels is set to a bad '
+                         'value, e.g., NaN or inf.')
+
+    # In Anderson & King (2000) notation this is psi_E(0.5, 0.0) and
+    # values used to compute derivatives.
+    psi_pos_x = data[yidx_0, xidx_0 + x_shiftidx]
+    psi_pos_x_m1 = data[yidx_0, xidx_0 + x_shiftidx - 1]
+    psi_pos_x_p1 = data[yidx_0, xidx_0 + x_shiftidx + 1]
+
+    # Our derivatives are simple differences across two data points, but
+    # this must be in units of the undersampled grid, so 2 pixels becomes
+    # 2/oversampling pixels
+    dpsi_pos_x = np.abs(psi_pos_x_p1 - psi_pos_x_m1) / (2. / oversampling[0])
+
+    # psi_E(-0.5, 0.0) and derivative components.
+    psi_neg_x = data[yidx_0, xidx_0 - x_shiftidx]
+    psi_neg_x_m1 = data[yidx_0, xidx_0 - x_shiftidx - 1]
+    psi_neg_x_p1 = data[yidx_0, xidx_0 - x_shiftidx + 1]
+    dpsi_neg_x = np.abs(psi_neg_x_p1 - psi_neg_x_m1) / (2. / oversampling[0])
+
+    x_shift = (psi_pos_x - psi_neg_x) / (dpsi_pos_x + dpsi_neg_x)
+
+    # psi_E(0.0, 0.5) and derivatives.
+    psi_pos_y = data[yidx_0 + y_shiftidx, xidx_0]
+    psi_pos_y_m1 = data[yidx_0 + y_shiftidx - 1, xidx_0]
+    psi_pos_y_p1 = data[yidx_0 + y_shiftidx + 1, xidx_0]
+    dpsi_pos_y = np.abs(psi_pos_y_p1 - psi_pos_y_m1) / (2. / oversampling[1])
+
+    # psi_E(0.0, -0.5) and derivative components.
+    psi_neg_y = data[yidx_0 - y_shiftidx, xidx_0]
+    psi_neg_y_m1 = data[yidx_0 - y_shiftidx - 1, xidx_0]
+    psi_neg_y_p1 = data[yidx_0 - y_shiftidx + 1, xidx_0]
+    dpsi_neg_y = np.abs(psi_neg_y_p1 - psi_neg_y_m1) / (2. / oversampling[1])
+
+    y_shift = (psi_pos_y - psi_neg_y) / (dpsi_pos_y + dpsi_neg_y)
+
+    return x_0 + x_shift, y_0 + y_shift
