@@ -89,8 +89,9 @@ def test_centroids_oversampling(xc_ref, yc_ref, x_stddev, y_stddev, theta):
         else:
             _oversampling = oversampling
         xc, yc = centroid_com(data, mask=mask, oversampling=oversampling)
-        assert_allclose([xc, yc], [xc_ref / _oversampling[0], yc_ref / _oversampling[1]],
-                        rtol=0, atol=1.e-3)
+        assert_allclose([xc, yc], [xc_ref / _oversampling[0],
+                                   yc_ref / _oversampling[1]], rtol=0,
+                        atol=1.e-3)
 
 
 @pytest.mark.skipif('not HAS_SCIPY')
@@ -153,7 +154,7 @@ def test_centroids_nan_withmask(use_mask):
 def test_centroid_com_mask():
     """Test centroid_com with and without an image_mask."""
 
-    data = np.ones((2, 2)).astype(np.float)
+    data = np.ones((2, 2)).astype(float)
     mask = [[False, False], [True, True]]
     centroid = centroid_com(data, mask=None)
     centroid_mask = centroid_com(data, mask=mask)
@@ -243,14 +244,37 @@ def test_centroid_epsf():
         y0 = y[-1] / 2
         y -= y0
         offsets = np.array([0.1, 0.03])
-        data = psf.evaluate(x=x.reshape(1, -1), y=y.reshape(-1, 1), flux=1, x_0=offsets[0],
-                            y_0=offsets[1], sigma=sigma)
+        data = psf.evaluate(x=x.reshape(1, -1), y=y.reshape(-1, 1), flux=1,
+                            x_0=offsets[0], y_0=offsets[1], sigma=sigma)
 
-        mask = np.zeros(data.shape, np.bool)
+        mask = np.zeros(data.shape, dtype=bool)
         mask[0, 0] = 1
         centers = centroid_epsf(data, mask=mask, oversampling=oversampling)
         assert_allclose(centers, offsets+x0, rtol=1e-3, atol=1e-2)
 
+    psf = Gaussian2D()
+    for oversampling in [4, (6, 2)]:
+        if not hasattr(oversampling, '__len__'):
+            _oversampling = (oversampling, oversampling)
+        else:
+            _oversampling = oversampling
+        x = np.arange(1 + 25 * _oversampling[0]) / _oversampling[0]
+        x0 = x[-1] / 2
+        x -= x0
+        y = np.arange(1 + 25 * _oversampling[1]) / _oversampling[1]
+        y0 = y[-1] / 2
+        y -= y0
+        offsets = np.array([0.1, 0.03])
+        data = psf.evaluate(x=x.reshape(1, -1), y=y.reshape(-1, 1),
+                            amplitude=1, x_mean=offsets[0], y_mean=offsets[1],
+                            x_stddev=5, y_stddev=0.5, theta=0)
+
+        mask = np.zeros(data.shape, dtype=bool)
+        mask[0, 0] = 1
+        centers = centroid_epsf(data, mask=mask, oversampling=oversampling,
+                                shift_val=0.5)
+        assert_allclose(centers, offsets+x0, rtol=1e-3, atol=1e-3)
+
 
 def test_centroid_exceptions():
     data = np.ones((5, 5), dtype=float)
