@@ -71,7 +71,7 @@ class _StarFinderKernel:
 
     References
     ----------
-    .. [1] http://en.wikipedia.org/wiki/Gaussian_function
+    .. [1] https://en.wikipedia.org/wiki/Gaussian_function
     """
 
     def __init__(self, fwhm, ratio=1.0, theta=0.0, sigma_radius=1.5,
@@ -128,7 +128,7 @@ class _StarFinderKernel:
 
         self.mask = np.where(
             (self.elliptical_radius <= self.f) |
-            (self.circular_radius <= 2.0), 1, 0).astype(np.int)
+            (self.circular_radius <= 2.0), 1, 0).astype(int)
         self.npixels = self.mask.sum()
 
         # NOTE: the central (peak) pixel of gaussian_kernel has a value of 1.
@@ -225,7 +225,7 @@ class _DAOFindProperties:
         calculate the source peak value, flux, and magnitude.  The
         default is 0.
 
-    .. _DAOFIND: http://stsdas.stsci.edu/cgi-bin/gethelp.cgi?daofind
+    .. _DAOFIND: https://iraf.net/irafhelp.php?val=daofind
     """
 
     def __init__(self, star_cutout, kernel, sky=0.):
@@ -492,7 +492,7 @@ class _IRAFStarFindProperties:
         self.kernel = kernel
 
         if sky is None:
-            skymask = ~self.kernel.mask.astype(np.bool)  # 1=sky, 0=obj
+            skymask = ~self.kernel.mask.astype(bool)  # 1=sky, 0=obj
             nsky = np.count_nonzero(skymask)
             if nsky == 0:
                 mean_sky = (np.max(self.cutout.data) -
@@ -623,8 +623,8 @@ def _find_stars(data, kernel, threshold_eff, min_separation=None,
         A list of `_StarCutout` objects containing the image cutout for
         each source.
 
-    .. _DAOFIND: http://stsdas.stsci.edu/cgi-bin/gethelp.cgi?daofind
-    .. _starfind: http://stsdas.stsci.edu/cgi-bin/gethelp.cgi?starfind
+    .. _DAOFIND: https://iraf.net/irafhelp.php?val=daofind
+    .. _starfind: https://iraf.net/irafhelp.php?val=starfind
     """
 
     convolved_data = _filter_data(data, kernel.data, mode='constant',
@@ -632,7 +632,7 @@ def _find_stars(data, kernel, threshold_eff, min_separation=None,
 
     # define a local footprint for the peak finder
     if min_separation is None:  # daofind
-        footprint = kernel.mask.astype(np.bool)
+        footprint = kernel.mask.astype(bool)
     else:
         # define a circular footprint
         idx = np.arange(-min_separation, min_separation + 1)
@@ -856,9 +856,9 @@ class DAOStarFinder(StarFinderBase):
     References
     ----------
     .. [1] Stetson, P. 1987; PASP 99, 191 (http://adsabs.harvard.edu/abs/1987PASP...99..191S)
-    .. [2] http://stsdas.stsci.edu/cgi-bin/gethelp.cgi?daofind
+    .. [2] https://iraf.net/irafhelp.php?val=daofind
 
-    .. _DAOFIND: http://stsdas.stsci.edu/cgi-bin/gethelp.cgi?daofind
+    .. _DAOFIND: https://iraf.net/irafhelp.php?val=daofind
     """
 
     def __init__(self, threshold, fwhm, ratio=1.0, theta=0.0,
@@ -1086,9 +1086,9 @@ class IRAFStarFinder(StarFinderBase):
 
     References
     ----------
-    .. [1] http://stsdas.stsci.edu/cgi-bin/gethelp.cgi?starfind
+    .. [1] https://iraf.net/irafhelp.php?val=starfind
 
-    .. _starfind: http://stsdas.stsci.edu/cgi-bin/gethelp.cgi?starfind
+    .. _starfind: https://iraf.net/irafhelp.php?val=starfind
     """
 
     def __init__(self, threshold, fwhm, sigma_radius=1.5, minsep_fwhm=2.5,
