@@ -1,84 +1,100 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
+"""
+Tests for the core module.
+"""
 
-import numpy as np
-from numpy.testing import assert_array_equal, assert_allclose
-import pytest
 import warnings
 
+from astropy.tests.helper import assert_quantity_allclose, catch_warnings
+import numpy as np
+from numpy.testing import assert_allclose, assert_array_equal
+import pytest
+
 from ..core import detect_threshold, find_peaks
 from ...centroids import centroid_com
-from ...datasets import make_4gaussians_image, make_wcs
+from ...datasets import make_4gaussians_image, make_gwcs, make_wcs
+from ...utils.exceptions import NoDetectionsWarning
 
 try:
-    import scipy    # noqa
+    import scipy  # noqa
     HAS_SCIPY = True
 except ImportError:
     HAS_SCIPY = False
 
-DATA = np.array([[0, 1, 0], [0, 2, 0], [0, 0, 0]]).astype(np.float)
+try:
+    # shared WCS interface requires gwcs >= 0.10
+    import gwcs  # noqa
+    HAS_GWCS = True
+except ImportError:
+    HAS_GWCS = False
+
+DATA = np.array([[0, 1, 0], [0, 2, 0], [0, 0, 0]]).astype(float)
 REF1 = np.array([[0, 0, 0], [0, 1, 0], [0, 0, 0]])
 
-PEAKDATA = np.array([[1, 0, 0], [0, 0, 0], [0, 0, 1]]).astype(np.float)
+PEAKDATA = np.array([[1, 0, 0], [0, 0, 0], [0, 0, 1]]).astype(float)
 PEAKREF1 = np.array([[0, 0], [2, 2]])
 
+IMAGE = make_4gaussians_image()
+FITSWCS = make_wcs(IMAGE.shape)
+
 
 @pytest.mark.skipif('not HAS_SCIPY')
 class TestDetectThreshold:
-    def test_snr(self):
-        """Test basic snr."""
+    def test_nsigma(self):
+        """Test basic nsigma."""
 
-        threshold = detect_threshold(DATA, snr=0.1)
+        threshold = detect_threshold(DATA, nsigma=0.1)
         ref = 0.4 * np.ones((3, 3))
         assert_allclose(threshold, ref)
 
-    def test_snr_zero(self):
-        """Test snr=0."""
+    def test_nsigma_zero(self):
+        """Test nsigma=0."""
 
-        threshold = detect_threshold(DATA, snr=0.0)
+        threshold = detect_threshold(DATA, nsigma=0.0)
         ref = (1. / 3.) * np.ones((3, 3))
         assert_allclose(threshold, ref)
 
     def test_background(self):
-        threshold = detect_threshold(DATA, snr=1.0, background=1)
+        threshold = detect_threshold(DATA, nsigma=1.0, background=1)
         ref = (5. / 3.) * np.ones((3, 3))
         assert_allclose(threshold, ref)
 
     def test_background_image(self):
         background = np.ones((3, 3))
-        threshold = detect_threshold(DATA, snr=1.0, background=background)
+        threshold = detect_threshold(DATA, nsigma=1.0, background=background)
         ref = (5. / 3.) * np.ones((3, 3))
         assert_allclose(threshold, ref)
 
     def test_background_badshape(self):
         wrong_shape = np.zeros((2, 2))
         with pytest.raises(ValueError):
-            detect_threshold(DATA, snr=2., background=wrong_shape)
+            detect_threshold(DATA, nsigma=2., background=wrong_shape)
 
     def test_error(self):
-        threshold = detect_threshold(DATA, snr=1.0, error=1)
+        threshold = detect_threshold(DATA, nsigma=1.0, error=1)
         ref = (4. / 3.) * np.ones((3, 3))
         assert_allclose(threshold, ref)
 
     def test_error_image(self):
         error = np.ones((3, 3))
-        threshold = detect_threshold(DATA, snr=1.0, error=error)
+        threshold = detect_threshold(DATA, nsigma=1.0, error=error)
         ref = (4. / 3.) * np.ones((3, 3))
         assert_allclose(threshold, ref)
 
     def test_error_badshape(self):
         wrong_shape = np.zeros((2, 2))
         with pytest.raises(ValueError):
-            detect_threshold(DATA, snr=2., error=wrong_shape)
+            detect_threshold(DATA, nsigma=2., error=wrong_shape)
 
     def test_background_error(self):
-        threshold = detect_threshold(DATA, snr=2.0, background=10., error=1.)
+        threshold = detect_threshold(DATA, nsigma=2.0, background=10., error=1.)
         ref = 12. * np.ones((3, 3))
         assert_allclose(threshold, ref)
 
     def test_background_error_images(self):
         background = np.ones((3, 3)) * 10.
         error = np.ones((3, 3))
-        threshold = detect_threshold(DATA, snr=2.0, background=background,
+        threshold = detect_threshold(DATA, nsigma=2.0, background=background,
                                      error=error)
         ref = 12. * np.ones((3, 3))
         assert_allclose(threshold, ref)
@@ -86,7 +102,7 @@ class TestDetectThreshold:
     def test_mask_value(self):
         """Test detection with mask_value."""
 
-        threshold = detect_threshold(DATA, snr=1.0, mask_value=0.0)
+        threshold = detect_threshold(DATA, nsigma=1.0, mask_value=0.0)
         ref = 2. * np.ones((3, 3))
         assert_array_equal(threshold, ref)
 
@@ -97,8 +113,8 @@ class TestDetectThreshold:
         applying the mask.
         """
 
-        mask = REF1.astype(np.bool)
-        threshold = detect_threshold(DATA, snr=1., error=0, mask=mask,
+        mask = REF1.astype(bool)
+        threshold = detect_threshold(DATA, nsigma=1., error=0, mask=mask,
                                      sigclip_sigma=10, sigclip_iters=1)
         ref = (1. / 8.) * np.ones((3, 3))
         assert_array_equal(threshold, ref)
@@ -106,8 +122,8 @@ class TestDetectThreshold:
     def test_image_mask_override(self):
         """Test that image_mask overrides mask_value."""
 
-        mask = REF1.astype(np.bool)
-        threshold = detect_threshold(DATA, snr=0.1, error=0, mask_value=0.0,
+        mask = REF1.astype(bool)
+        threshold = detect_threshold(DATA, nsigma=0.1, error=0, mask_value=0.0,
                                      mask=mask, sigclip_sigma=10,
                                      sigclip_iters=1)
         ref = np.ones((3, 3))
@@ -132,21 +148,10 @@ class TestFindPeaks:
         assert_array_equal(tbl['y_peak'], PEAKREF1[:, 0])
         assert_array_equal(tbl['peak_value'], [1., 1.])
 
-    def test_centroid_func_and_subpixel(self):
-        with pytest.raises(ValueError):
-            find_peaks(PEAKDATA, 0.1, centroid_func=centroid_com,
-                       subpixel=True)
-
-    def test_subpixel_regionsize(self):
-        """Test that data cutout has at least 6 values."""
-
-        with pytest.raises(ValueError):
-            find_peaks(PEAKDATA, 0.1, box_size=2, subpixel=True)
-
     def test_mask(self):
         """Test with mask."""
 
-        mask = np.zeros_like(PEAKDATA, dtype=bool)
+        mask = np.zeros(PEAKDATA.shape, dtype=bool)
         mask[0, 0] = True
         tbl = find_peaks(PEAKDATA, 0.1, box_size=3, mask=mask)
         assert len(tbl) == 1
@@ -175,9 +180,13 @@ class TestFindPeaks:
 
     def test_border_width(self):
         """Test border exclusion."""
+        with catch_warnings(NoDetectionsWarning) as warning_lines:
+            tbl = find_peaks(PEAKDATA, 0.1, box_size=3, border_width=3)
+            assert tbl is None
 
-        tbl = find_peaks(PEAKDATA, 0.1, box_size=3, border_width=3)
-        assert len(tbl) == 0
+            assert warning_lines[0].category == NoDetectionsWarning
+            assert ('No local peaks were found.' in
+                    str(warning_lines[0].message))
 
     def test_box_size_int(self):
         """Test non-integer box_size."""
@@ -189,31 +198,51 @@ class TestFindPeaks:
     def test_centroid_func_callable(self):
         """Test that centroid_func is callable."""
 
-        with pytest.raises(ValueError):
+        with pytest.raises(TypeError):
             find_peaks(PEAKDATA, 0.1, box_size=2, centroid_func=True)
 
     def test_wcs(self):
-        """Test with WCS."""
+        """Test with astropy WCS."""
 
-        data = make_4gaussians_image()
-        wcs = make_wcs(data.shape)
-        cols = ['skycoord_peak', 'skycoord_centroid']
+        columns = ['skycoord_peak', 'skycoord_centroid']
 
-        tbl = find_peaks(data, 100, wcs=wcs, centroid_func=centroid_com)
-        for col in cols:
-            assert col in tbl.colnames
+        tbl = find_peaks(IMAGE, 100, wcs=FITSWCS, centroid_func=centroid_com)
+        for column in columns:
+            assert column in tbl.colnames
 
-        tbl = find_peaks(data, 100, wcs=wcs, subpixel=True)
-        for col in cols:
-            assert col in tbl.colnames
+    @pytest.mark.skipif('not HAS_GWCS')
+    def test_gwcs(self):
+        """Test with gwcs."""
+
+        columns = ['skycoord_peak', 'skycoord_centroid']
+
+        gwcs_obj = make_gwcs(IMAGE.shape)
+        tbl = find_peaks(IMAGE, 100, wcs=gwcs_obj, centroid_func=centroid_com)
+        for column in columns:
+            assert column in tbl.colnames
+
+    @pytest.mark.skipif('not HAS_GWCS')
+    def test_wcs_values(self):
+        gwcs_obj = make_gwcs(IMAGE.shape)
+        tbl1 = find_peaks(IMAGE, 100, wcs=FITSWCS, centroid_func=centroid_com)
+        tbl2 = find_peaks(IMAGE, 100, wcs=gwcs_obj,
+                          centroid_func=centroid_com)
+        columns = ['skycoord_peak', 'skycoord_centroid']
+        for column in columns:
+            assert_quantity_allclose(tbl1[column].ra, tbl2[column].ra)
+            assert_quantity_allclose(tbl1[column].dec, tbl2[column].dec)
 
     def test_constant_array(self):
         """Test for empty output table when data is constant."""
 
         data = np.ones((10, 10))
-        tbl = find_peaks(data, 0.)
-        assert len(tbl) == 0
-        assert set(tbl.colnames) == {'x_peak', 'y_peak', 'peak_value'}
+        with catch_warnings(NoDetectionsWarning) as warning_lines:
+            tbl = find_peaks(data, 0.)
+            assert tbl is None
+
+            assert warning_lines[0].category == NoDetectionsWarning
+            assert ('Input data is constant.' in
+                    str(warning_lines[0].message))
 
     def test_no_peaks(self):
         """
@@ -221,32 +250,18 @@ class TestFindPeaks:
         when no peaks are found.
         """
 
-        data = make_4gaussians_image()
-        wcs = make_wcs(data.shape)
+        tbl = find_peaks(IMAGE, 10000)
+        assert tbl is None
 
-        tbl1 = find_peaks(data, 100)
-        tbl2 = find_peaks(data, 10000)
-        assert set(tbl1.colnames) == set(tbl2.colnames)
+        tbl = find_peaks(IMAGE, 100000, centroid_func=centroid_com)
+        assert tbl is None
 
-        tbl1 = find_peaks(data, 100, centroid_func=centroid_com)
-        tbl2 = find_peaks(data, 100000, centroid_func=centroid_com)
-        assert set(tbl1.colnames) == set(tbl2.colnames)
+        tbl = find_peaks(IMAGE, 100000, wcs=FITSWCS)
+        assert tbl is None
 
-        tbl1 = find_peaks(data, 100, subpixel=True)
-        tbl2 = find_peaks(data, 100000, subpixel=True)
-        assert set(tbl1.colnames) == set(tbl2.colnames)
-
-        tbl1 = find_peaks(data, 100, wcs=wcs)
-        tbl2 = find_peaks(data, 100000, wcs=wcs)
-        assert set(tbl1.colnames) == set(tbl2.colnames)
-
-        tbl1 = find_peaks(data, 100, wcs=wcs, centroid_func=centroid_com)
-        tbl2 = find_peaks(data, 100000, wcs=wcs, centroid_func=centroid_com)
-        assert set(tbl1.colnames) == set(tbl2.colnames)
-
-        tbl1 = find_peaks(data, 100, wcs=wcs, subpixel=True)
-        tbl2 = find_peaks(data, 100000, wcs=wcs, subpixel=True)
-        assert set(tbl1.colnames) == set(tbl2.colnames)
+        tbl = find_peaks(IMAGE, 100000, wcs=FITSWCS,
+                         centroid_func=centroid_com)
+        assert tbl is None
 
     def test_data_nans(self):
         """Test that data with NaNs does not issue Runtime warning."""
