@@ -1,18 +1,20 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
+"""
+This module provides a class to fit elliptical isophotes.
+"""
 
 import warnings
 
-import numpy as np
 from astropy.utils.exceptions import AstropyUserWarning
+import numpy as np
 
-from .fitter import (EllipseFitter, CentralEllipseFitter,
-                     DEFAULT_CONVERGENCE, DEFAULT_MINIT, DEFAULT_MAXIT,
-                     DEFAULT_FFLAG, DEFAULT_MAXGERR)
+from .fitter import (DEFAULT_CONVERGENCE, DEFAULT_FFLAG, DEFAULT_MAXGERR,
+                     DEFAULT_MAXIT, DEFAULT_MINIT, CentralEllipseFitter,
+                     EllipseFitter)
 from .geometry import EllipseGeometry
 from .integrator import BILINEAR
 from .isophote import Isophote, IsophoteList
-from .sample import EllipseSample, CentralEllipseSample
-
+from .sample import CentralEllipseSample, EllipseSample
 
 __all__ = ['Ellipse']
 
@@ -183,6 +185,7 @@ class Ellipse:
             _y0 = image.shape[0] / 2
             self._geometry = EllipseGeometry(_x0, _y0, 10., eps=0.2,
                                              pa=np.pi/2)
+        self.set_threshold(threshold)
 
     def set_threshold(self, threshold):
         """
@@ -200,7 +203,8 @@ class Ellipse:
                   conver=DEFAULT_CONVERGENCE, minit=DEFAULT_MINIT,
                   maxit=DEFAULT_MAXIT, fflag=DEFAULT_FFLAG,
                   maxgerr=DEFAULT_MAXGERR, sclip=3., nclip=0,
-                  integrmode=BILINEAR, linear=False, maxrit=None):
+                  integrmode=BILINEAR, linear=None, maxrit=None,
+                  fix_center=False, fix_pa=False, fix_eps=False):
         # This parameter list is quite large and should in principle be
         # simplified by re-distributing these controls to somewhere else.
         # We keep this design though because it better mimics the flat
@@ -213,6 +217,15 @@ class Ellipse:
         isophote at each sma.  The entire set of isophotes is returned
         in an `~photutils.isophote.IsophoteList` instance.
 
+        Note that the fix_XXX parameters act in unison. Meaning, if one
+        of them is set via this call, the others will assume their default
+        (False) values. This effectively overrides any settings that are
+        present in the internal `~photutils.isophote.EllipseGeometry`
+        instance that is carried along as a property of this class. If an
+        instance of `~photutils.isophote.EllipseGeometry` was passed to this
+        class' constructor, that instance will be effectively overriden by
+        the fix_XXX parameters in this call.
+
         Parameters
         ----------
         sma0 : float, optional
@@ -331,6 +344,13 @@ class Ellipse:
             whenever the ellipticity exceeds 1.0 or the ellipse center
             crosses the image boundaries.  If `None` (default), then no
             maximum value is used.
+        fix_center : bool, optional
+            Keep center of ellipse fixed during fit? The default is False.
+        fix_pa : bool, optional
+            Keep position angle of semi-major axis of ellipse fixed during fit?
+            The default is False.
+        fix_eps : bool, optional
+            Keep ellipticity of ellipse fixed during fit? The default is False.
 
         Returns
         -------
@@ -352,6 +372,19 @@ class Ellipse:
         else:
             sma = sma0
 
+        # Override geometry instance with parameters set at the call.
+        if isinstance(linear, bool):
+            self._geometry.linear_growth = linear
+        else:
+            linear = self._geometry.linear_growth
+        if fix_center and fix_pa and fix_eps:
+            warnings.warn(': Everything is fixed. Fit not possible.',
+                          AstropyUserWarning)
+            return IsophoteList([])
+        if fix_center or fix_pa or fix_eps:
+            # Note that this overrides the geometry instance for good.
+            self._geometry.fix = np.array([fix_center, fix_center, fix_pa, fix_eps])
+
         # first, go from initial sma outwards until
         # hitting one of several stopping criteria.
         noiter = False
@@ -441,7 +474,8 @@ class Ellipse:
 
         # if user asked for minsma=0, extract special isophote there
         if minsma == 0.0:
-            isophote = self.fit_isophote(0.0, isophote_list=isophote_list)
+            # isophote is appended to isophote_list
+            _ = self.fit_isophote(0.0, isophote_list=isophote_list)
 
         # sort list of isophotes according to sma
         isophote_list.sort()
@@ -560,7 +594,7 @@ class Ellipse:
 
         # if available, geometry from last fitted isophote will be
         # used as initial guess for next isophote.
-        if isophote_list is not None and len(isophote_list) > 0:
+        if isophote_list:
             geometry = isophote_list[-1].sample.geometry
 
         # do the fit
@@ -602,15 +636,16 @@ class Ellipse:
         sample = EllipseSample(self.image, sma, astep=step, sclip=sclip,
                                nclip=nclip, linear_growth=linear,
                                geometry=geometry, integrmode=integrmode)
-        sample.update()
+        sample.update(geometry.fix)
 
         # build isophote without iterating with an EllipseFitter
         isophote = Isophote(sample, 0, True, stop_code=4)
 
         return isophote
 
-    def _fix_last_isophote(self, isophote_list, index):
-        if len(isophote_list) > 0:
+    @staticmethod
+    def _fix_last_isophote(isophote_list, index):
+        if isophote_list:
             isophote = isophote_list.pop()
 
             # check if isophote is bad; if so, fix its geometry
@@ -621,7 +656,7 @@ class Ellipse:
             # force new extraction of raw data, since
             # geometry changed.
             isophote.sample.values = None
-            isophote.sample.update()
+            isophote.sample.update(isophote.sample.geometry.fix)
 
             # we take the opportunity to change an eventual
             # negative stop code to its' positive equivalent.
