@@ -1,16 +1,21 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
+"""
+This module provides a class to fit ellipses.
+"""
+
+import math
 
 from astropy import log
-import math
 import numpy as np
+import numpy.ma as ma
 
-from .harmonics import (fit_first_and_second_harmonics,
-                        first_and_second_harmonic_function)
-from .isophote import Isophote, CentralPixel
+from .harmonics import (first_and_second_harmonic_function,
+                        fit_first_and_second_harmonics)
+from .isophote import CentralPixel, Isophote
 from .sample import EllipseSample
 
-
 __all__ = ['EllipseFitter']
+
 __doctest_skip__ = ['EllipseFitter.fit']
 
 
@@ -134,9 +139,12 @@ class EllipseFitter:
         minimum_amplitude_value = np.Inf
         minimum_amplitude_sample = None
 
-        for iter in range(maxit):
+        # these must be passed throughout the execution chain.
+        fixed_parameters = self._sample.geometry.fix
+
+        for i in range(maxit):
             # Force the sample to compute its gradient and associated values.
-            sample.update()
+            sample.update(fixed_parameters)
 
             # The extract() method returns sampled values as a 2-d numpy array
             # with the following structure:
@@ -150,15 +158,18 @@ class EllipseFitter:
             # marked as invalid.
             try:
                 coeffs = fit_first_and_second_harmonics(values[0], values[2])
+                coeffs = coeffs[0]
             except Exception as e:
                 log.info(e)
-                return Isophote(sample, iter+1, False, 3)
+                sample.geometry.fix = fixed_parameters
+                return Isophote(sample, i + 1, False, 3)
 
-            coeffs = coeffs[0]
+            # Mask out coefficients that control fixed ellipse parameters.
+            free_coeffs = ma.masked_array(coeffs[1:], mask=fixed_parameters)
 
-            # largest harmonic in absolute value drives the correction.
-            largest_harmonic_index = np.argmax(np.abs(coeffs[1:]))
-            largest_harmonic = coeffs[1:][largest_harmonic_index]
+            # Largest non-masked harmonic in absolute value drives the correction.
+            largest_harmonic_index = np.argmax(np.abs(free_coeffs))
+            largest_harmonic = free_coeffs[largest_harmonic_index]
 
             # see if the amplitude decreased; if yes, keep the
             # corresponding sample for eventual later use.
@@ -174,20 +185,20 @@ class EllipseFitter:
                     > np.abs(largest_harmonic)):
                 # Got a valid solution. But before returning, ensure
                 # that a minimum of iterations has run.
-                if iter >= minit-1:
-                    sample.update()
-                    return Isophote(sample, iter+1, True, 0)
+                if i >= minit - 1:
+                    sample.update(fixed_parameters)
+                    return Isophote(sample, i + 1, True, 0)
 
             # it may not have converged yet, but the sample contains too
             # many invalid data points: return.
             if sample.actual_points < (sample.total_points * fflag):
                 # when too many data points were flagged, return the
                 # best fit sample instead of the current one.
-                minimum_amplitude_sample.update()
-                return Isophote(minimum_amplitude_sample, iter+1, True, 1)
+                minimum_amplitude_sample.update(fixed_parameters)
+                return Isophote(minimum_amplitude_sample, i + 1, True, 1)
 
             # pick appropriate corrector code.
-            corrector = _correctors[largest_harmonic_index]
+            corrector = _CORRECTORS[largest_harmonic_index]
 
             # generate *NEW* EllipseSample instance with corrected
             # parameter.  Note that this instance is still devoid of other
@@ -199,7 +210,7 @@ class EllipseFitter:
             # (hopefully smaller) price here, by having multiple calls to
             # the EllipseSample constructor.
             sample = corrector.correct(sample, largest_harmonic)
-            sample.update()
+            sample.update(fixed_parameters)
 
             # see if any abnormal (or unusual) conditions warrant
             # the change to non-iterative mode, or go-inwards mode.
@@ -207,16 +218,17 @@ class EllipseFitter:
                 sample, maxgerr, going_inwards, lexceed)
 
             if not proceed:
-                sample.update()
-                return Isophote(sample, iter+1, True, -1)
+                sample.update(fixed_parameters)
+                return Isophote(sample, i + 1, True, -1)
 
         # Got to the maximum number of iterations. Return with
         # code 2, and handle it as a valid isophote. Use the
         # best fit sample instead of the current one.
-        minimum_amplitude_sample.update()
+        minimum_amplitude_sample.update(fixed_parameters)
         return Isophote(minimum_amplitude_sample, maxit, True, 2)
 
-    def _check_conditions(self, sample, maxgerr, going_inwards, lexceed):
+    @staticmethod
+    def _check_conditions(sample, maxgerr, going_inwards, lexceed):
         proceed = True
 
         # If center wandered more than allowed, put it back
@@ -230,9 +242,9 @@ class EllipseFitter:
 
         # check if an acceptable gradient value could be computed.
         if sample.gradient_error:
-            if (not going_inwards and
-                (sample.gradient_relative_error > maxgerr or
-                 sample.gradient >= 0.)):
+            if not going_inwards and (
+                    sample.gradient_relative_error > maxgerr
+                    or sample.gradient >= 0.0):
                 if lexceed:
                     proceed = False
                 else:
@@ -274,7 +286,8 @@ class _ParameterCorrector:
 
 class _PositionCorrector(_ParameterCorrector):
 
-    def finalize_correction(self, dx, dy, sample):
+    @staticmethod
+    def finalize_correction(dx, dy, sample):
         new_x0 = sample.geometry.x0 + dx
         new_y0 = sample.geometry.y0 + dy
 
@@ -287,7 +300,7 @@ class _PositionCorrector(_ParameterCorrector):
                              integrmode=sample.integrmode)
 
 
-class _PositionCorrector_0(_PositionCorrector):
+class _PositionCorrector0(_PositionCorrector):
 
     def correct(self, sample, harmonic):
         aux = -harmonic * (1. - sample.geometry.eps) / sample.gradient
@@ -298,7 +311,7 @@ class _PositionCorrector_0(_PositionCorrector):
         return self.finalize_correction(dx, dy, sample)
 
 
-class _PositionCorrector_1(_PositionCorrector):
+class _PositionCorrector1(_PositionCorrector):
 
     def correct(self, sample, harmonic):
         aux = -harmonic / sample.gradient
@@ -352,7 +365,7 @@ class _EllipticityCorrector(_ParameterCorrector):
 
 
 # instances of corrector code live here:
-_correctors = [_PositionCorrector_0(), _PositionCorrector_1(),
+_CORRECTORS = [_PositionCorrector0(), _PositionCorrector1(),
                _AngleCorrector(), _EllipticityCorrector()]
 
 
@@ -382,6 +395,8 @@ class CentralEllipseFitter(EllipseFitter):
             at the central position.  Thus, most of its attributes are
             hardcoded to `None` or other default value when appropriate.
         """
+        # default values
+        fixed_parameters = np.array([False, False, False, False])
 
-        self._sample.update()
+        self._sample.update(fixed_parameters)
         return CentralPixel(self._sample)
