@@ -1,11 +1,13 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
+"""
+This module provides a container class to store parameters for the
+geometry of an ellipse.
+"""
 
 import math
-import numpy as np
-from astropy import log
 
+from astropy import log
+import numpy as np
 
 __all__ = ['EllipseGeometry']
 
@@ -61,7 +63,7 @@ def _area(sma, eps, phi, r):
     return abs(sma**2 * (1.-eps) / 2. * math.acos(aux))
 
 
-class EllipseGeometry(object):
+class EllipseGeometry:
     """
     Container class to store parameters for the geometry of an ellipse.
 
@@ -102,11 +104,19 @@ class EllipseGeometry(object):
         as a relative value (when ``linear_growth=False``).  The default
         is 0.1.
     linear_growth : bool, optional
-        The semimajor axis growing/shrinking mode.  The default is
+        The semimajor axis growing/shrinking mode. The default is
         `False`.
+    fix_center : bool, optional
+        Keep center of ellipse fixed during fit? The default is False.
+    fix_pa : bool, optional
+        Keep position angle of semi-major axis of ellipse fixed during fit?
+        The default is False.
+    fix_eps : bool, optional
+        Keep ellipticity of ellipse fixed during fit? The default is False.
     """
 
-    def __init__(self, x0, y0, sma, eps, pa, astep=0.1, linear_growth=False):
+    def __init__(self, x0, y0, sma, eps, pa, astep=0.1, linear_growth=False,
+                 fix_center=False, fix_pa=False, fix_eps=False):
         self.x0 = x0
         self.y0 = y0
         self.sma = sma
@@ -116,6 +126,11 @@ class EllipseGeometry(object):
         self.astep = astep
         self.linear_growth = linear_growth
 
+        # Fixed parameters are flagged in here. Note that the
+        # ordering must follow the same ordering used in the
+        # fitter._CORRECTORS list.
+        self.fix = np.array([fix_center, fix_center, fix_pa, fix_eps])
+
         # limits for sector angular width
         self._phi_min = 0.05
         self._phi_max = 0.2
@@ -200,10 +215,10 @@ class EllipseGeometry(object):
         shape = image.shape
         _x0 = self.x0
         _y0 = self.y0
-        if (_x0 is None or _x0 < 0 or _x0 >= shape[0] or _y0 is None or
-                _y0 < 0 or _y0 >= shape[1]):
-            _x0 = shape[0] / 2
-            _y0 = shape[1] / 2
+        if (_x0 is None or _x0 < 0 or _x0 >= shape[1] or _y0 is None or
+                _y0 < 0 or _y0 >= shape[0]):
+            _x0 = shape[1] / 2
+            _y0 = shape[0] / 2
 
         max_fom = 0.
         max_i = 0
@@ -219,8 +234,8 @@ class EllipseGeometry(object):
                 # ensure that it stays inside image frame
                 i1 = int(max(0, i - self._centerer_mask_half_size))
                 j1 = int(max(0, j - self._centerer_mask_half_size))
-                i2 = int(min(shape[0] - 1, i + self._centerer_mask_half_size))
-                j2 = int(min(shape[1] - 1, j + self._centerer_mask_half_size))
+                i2 = int(min(shape[1] - 1, i + self._centerer_mask_half_size))
+                j2 = int(min(shape[0] - 1, j + self._centerer_mask_half_size))
 
                 window = image[j1:j2, i1:i2]
 
@@ -357,7 +372,7 @@ class EllipseGeometry(object):
             define the annulus bounding ellipses.
         """
 
-        if (self.linear_growth):
+        if self.linear_growth:
             a1 = self.sma - self.astep / 2.
             a2 = self.sma + self.astep / 2.
         else:
@@ -411,6 +426,50 @@ class EllipseGeometry(object):
         radius, angle : float
             The ellipse radius and polar angle.
         """
+        # We split in between a scalar version and a
+        # vectorized version. This is necessary for
+        # now so we don't pay a heavy speed penalty
+        # that is incurred when using vectorized code.
+
+        # The split in two separate functions helps in
+        # the profiling analysis: most of the time is
+        # spent in the scalar function.
+
+        if isinstance(x, (int, float)):
+            return self._to_polar_scalar(x, y)
+        else:
+            return self._to_polar_vectorized(x, y)
+
+    def _to_polar_scalar(self, x, y):
+
+        x1 = x - self.x0
+        y1 = y - self.y0
+
+        radius = x1**2 + y1**2
+        if radius > 0.0:
+            radius = math.sqrt(radius)
+            angle = math.asin(abs(y1) / radius)
+        else:
+            radius = 0.
+            angle = 1.
+
+        if x1 >= 0. and y1 < 0.:
+            angle = 2*np.pi - angle
+        elif x1 < 0. and y1 >= 0.:
+            angle = np.pi - angle
+        elif x1 < 0. and y1 < 0.:
+            angle = np.pi + angle
+
+        pa1 = self.pa
+        if self.pa < 0.:
+            pa1 = self.pa + 2*np.pi
+        angle = angle - pa1
+        if angle < 0.:
+            angle = angle + 2*np.pi
+
+        return radius, angle
+
+    def _to_polar_vectorized(self, x, y):
 
         x1 = np.atleast_2d(x) - self.x0
         y1 = np.atleast_2d(y) - self.y0
