@@ -1,12 +1,14 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
+"""
+This module provides a class to sample data along an elliptical path.
+"""
 
 import copy
 
 import numpy as np
 
 from .geometry import EllipseGeometry
-from .integrator import integrators
-
+from .integrator import INTEGRATORS
 
 __all__ = ['EllipseSample']
 
@@ -173,7 +175,7 @@ class EllipseSample:
         self.actual_points = 0
 
         # build integrator
-        integrator = integrators[self.integrmode](self.image, self.geometry,
+        integrator = INTEGRATORS[self.integrmode](self.image, self.geometry,
                                                   angles, radii, intensities)
 
         # initialize walk along elliptical path
@@ -196,10 +198,10 @@ class EllipseSample:
             radii = []
             intensities = []
             if area < 1.0:
-                integrator = integrators['bilinear'](
+                integrator = INTEGRATORS['bilinear'](
                     self.image, self.geometry, angles, radii, intensities)
             else:
-                integrator = integrators[self.integrmode](self.image,
+                integrator = INTEGRATORS[self.integrmode](self.image,
                                                           self.geometry,
                                                           angles, radii,
                                                           intensities)
@@ -207,7 +209,7 @@ class EllipseSample:
         # walk along elliptical path, integrating at specified
         # places defined by polar vector. Need to go a bit beyond
         # full circle to ensure full coverage.
-        while (phi <= np.pi*2. + phi_min):
+        while phi <= np.pi*2. + phi_min:
             # do the integration at phi-radius position, and append
             # results to the angles, radii, and intensities lists.
             integrator.integrate(radius, phi)
@@ -243,7 +245,7 @@ class EllipseSample:
 
     def _sigma_clip(self, angles, radii, intensities):
         if self.nclip > 0:
-            for iter in range(self.nclip):
+            for i in range(self.nclip):
                 # do not use list.copy()! must be python2-compliant.
                 angles, radii, intensities = self._iter_sigma_clip(
                     angles[:], radii[:], intensities[:])
@@ -276,7 +278,7 @@ class EllipseSample:
 
         return r_angles, r_radii, r_intensities
 
-    def update(self):
+    def update(self, fixed_parameters):
         """
         Update this `~photutils.isophote.EllipseSample` instance.
 
@@ -286,6 +288,7 @@ class EllipseSample:
         then computes the the mean intensity, local gradient, and other
         associated quantities.
         """
+        self.geometry.fix = fixed_parameters
 
         step = self.geometry.astep
 
@@ -301,12 +304,16 @@ class EllipseSample:
         # another sample, this time using larger radius. Meaningful
         # gradient means something  shallower, but still close to within
         # a factor 3 from previous gradient estimate. If no previous
-        # estimate is available, guess it.
+        # estimate is available, guess it by adding the error to the
+        # current gradient.
         previous_gradient = self.gradient
         if not previous_gradient:
-            previous_gradient = -0.05    # good enough, based on usage
+            previous_gradient = gradient + gradient_error
 
-        if gradient >= (previous_gradient / 3.):   # gradient is negative!
+            # solution adopted before 08/12/2019
+            # previous_gradient = -0.05  # good enough, based on usage
+
+        if gradient >= (previous_gradient / 3.):  # gradient is negative!
             gradient, gradient_error = self._get_gradient(2 * step)
 
         # If still no meaningful gradient can be measured, try with
@@ -383,11 +390,13 @@ class CentralEllipseSample(EllipseSample):
     the special case of the central pixel in the galaxy image.
     """
 
-    def update(self):
+    def update(self, fixed_parameters):
         """
         Update this `~photutils.isophote.EllipseSample` instance with
         the intensity integrated at the (x0, y0) center position using
         bilinear integration. The local gradient is set to `None`.
+
+        'fixed_parameters' is ignored in this subclass.
         """
 
         s = self.extract()
@@ -402,7 +411,7 @@ class CentralEllipseSample(EllipseSample):
         radii = []
         intensities = []
 
-        integrator = integrators['bilinear'](self.image, self.geometry,
+        integrator = INTEGRATORS['bilinear'](self.image, self.geometry,
                                              angles, radii, intensities)
         integrator.integrate(0.0, 0.0)
 
