@@ -1,8 +1,10 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-
-import numpy as np
+"""
+This module provides tools for making a simulated image for tests.
+"""
 
 from astropy.io import fits
+import numpy as np
 
 from ..geometry import EllipseGeometry
 from ...datasets import make_noise_image
@@ -32,7 +34,7 @@ def make_test_image(nx=512, ny=512, x0=None, y0=None,
         The ellipticity of the reference isophote.
     pa : float, optional
         The position angle of the reference isophote.
-    random_state : int or `~numpy.random.RandomState`, optional
+    random_state : int or `~numpy.random.mtrand.RandomState`, optional
         Pseudo-random number generator state used for random sampling.
         Separate function calls with the same ``random_state`` will
         generate the identical noise image.
@@ -65,7 +67,7 @@ def make_test_image(nx=512, ny=512, x0=None, y0=None,
                                    image[int(xcen), int(ycen - 1)] +
                                    image[int(xcen), int(ycen + 1)]) / 4.
 
-    image += make_noise_image(image.shape, type='gaussian', mean=0.,
+    image += make_noise_image(image.shape, distribution='gaussian', mean=0.,
                               stddev=noise, random_state=random_state)
 
     return image
@@ -94,7 +96,7 @@ def make_fits_test_image(name, nx=512, ny=512, x0=None, y0=None,
         name += '.fits'
 
     array = make_test_image(nx, ny, x0, y0, background, noise, i0, sma, eps,
-                            pa)
+                            pa, random_state=random_state)
 
     hdu = fits.PrimaryHDU(array)
     hdulist = fits.HDUList([hdu])
