@@ -1,10 +1,13 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
+"""
+Tests for the ellipse module.
+"""
 
-import numpy as np
-import pytest
-
+import math
 from astropy.io import fits
 from astropy.modeling.models import Gaussian2D
+import numpy as np
+import pytest
 
 from .make_test_data import make_test_image
 from ..ellipse import Ellipse
@@ -12,9 +15,8 @@ from ..geometry import EllipseGeometry
 from ..isophote import Isophote, IsophoteList
 from ...datasets import get_path, make_noise_image
 
-
 try:
-    import scipy    # noqa
+    import scipy  # noqa
     HAS_SCIPY = True
 except ImportError:
     HAS_SCIPY = False
@@ -35,7 +37,7 @@ OFFSET_GALAXY = make_test_image(x0=POS, y0=POS, pa=PA, noise=1.e-12,
 @pytest.mark.skipif('not HAS_SCIPY')
 class TestEllipse:
     def setup_class(self):
-        # centered, tilted galaxy.
+        # centered, tilted galaxy
         self.data = make_test_image(pa=PA, random_state=123)
 
     @pytest.mark.remote_data
@@ -62,9 +64,25 @@ class TestEllipse:
         # verify that the list is properly sorted in sem-major axis length
         assert isophote_list[-1] > isophote_list[0]
 
-        # the fit should stop where gradient looses reliability.
-        assert len(isophote_list) == 67
-        assert isophote_list[-1].stop_code == 5
+        # the fit should stop where gradient loses reliability.
+        assert len(isophote_list) == 69
+        assert isophote_list[-1].stop_code == 1
+
+    def test_linear(self):
+        ellipse = Ellipse(self.data)
+        isophote_list = ellipse.fit_image(linear=True, step=2.)
+
+        # verify that the list is properly sorted in sem-major axis length
+        assert isophote_list[-1] > isophote_list[0]
+
+        # difference in sma between successive isohpotes must be constant.
+        step = isophote_list[-1].sma - isophote_list[-2].sma
+        assert math.isclose((isophote_list[-2].sma - isophote_list[-3].sma),
+                            step, rel_tol=0.01)
+        assert math.isclose((isophote_list[-3].sma - isophote_list[-4].sma),
+                            step, rel_tol=0.01)
+        assert math.isclose((isophote_list[2].sma - isophote_list[1].sma),
+                            step, rel_tol=0.01)
 
     def test_fit_one_ellipse(self):
         ellipse = Ellipse(self.data)
@@ -117,10 +135,10 @@ class TestEllipse:
         nx = 150
         g = Gaussian2D(100., nx / 2., ny / 2., 20, 12, theta=40.*np.pi/180.)
         y, x = np.mgrid[0:ny, 0:nx]
-        noise = make_noise_image((ny, nx), type='gaussian', mean=0.,
+        noise = make_noise_image((ny, nx), distribution='gaussian', mean=0.,
                                  stddev=2., random_state=12345)
         data = g(x, y) + noise
-        ellipse = Ellipse(data)    # estimates initial center
+        ellipse = Ellipse(data)  # estimates initial center
         isolist = ellipse.fit_image()
         assert len(isolist) == 54
 
@@ -140,7 +158,7 @@ class TestEllipseOnRealData:
         ellipse = Ellipse(data, geometry=g)
         isophote_list = ellipse.fit_image()
 
-        assert len(isophote_list) == 57
+        assert len(isophote_list) >= 60
 
         # check that isophote at about sma=70 got an uneventful fit
         assert isophote_list.get_closest(70.).stop_code == 0
