@@ -1,13 +1,17 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
+"""
+Tests for the sample module.
+"""
 
 import numpy as np
 import pytest
 
 from .make_test_data import make_test_image
-from ..integrator import MEDIAN, MEAN, BILINEAR, NEAREST_NEIGHBOR
+from ..integrator import BILINEAR, MEAN, MEDIAN, NEAREST_NEIGHBOR
 from ..isophote import Isophote
 from ..sample import EllipseSample
 
+DEFAULT_FIX = np.array([False, False, False, False])
 
 DATA = make_test_image(background=100., i0=0., noise=10., random_state=123)
 
@@ -29,7 +33,7 @@ def test_scatter(integrmode, amin, amax):
     """
 
     sample = EllipseSample(DATA, 50., astep=0.2, integrmode=integrmode)
-    sample.update()
+    sample.update(DEFAULT_FIX)
     iso = Isophote(sample, 0, True, 0)
 
     assert iso.pix_stddev < amax
@@ -38,7 +42,7 @@ def test_scatter(integrmode, amin, amax):
 
 def test_coordinates():
     sample = EllipseSample(DATA, 50.)
-    sample.update()
+    sample.update(DEFAULT_FIX)
     x, y = sample.coordinates()
 
     assert isinstance(x, np.ndarray)
@@ -47,7 +51,7 @@ def test_coordinates():
 
 def test_sclip():
     sample = EllipseSample(DATA, 50., nclip=3)
-    sample.update()
+    sample.update(DEFAULT_FIX)
     x, y = sample.coordinates()
 
     assert isinstance(x, np.ndarray)
