@@ -1,10 +1,13 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 """
-Functions for measuring non-parametric morphology.
+This module provides functions for measuring non-parametric morphologies
+of sources.
 """
 
 import numpy as np
 
+__all__ = ['gini']
+
 
 def gini(data):
     """
@@ -24,10 +27,11 @@ def gini(data):
 
     The Gini coefficient is a way of measuring the inequality in a given
     set of values. In the context of galaxy morphology, it measures how
-    the light of a galaxy image is distributed among its pixels.  A
-    ``G`` value of 0 corresponds to a galaxy image with the light evenly
-    distributed over all pixels while a ``G`` value of 1 represents a
-    galaxy image with all its light concentrated in just one pixel.
+    the light of a galaxy image is distributed among its pixels.  A Gini
+    coefficient value of 0 corresponds to a galaxy image with the light
+    evenly distributed over all pixels while a Gini coefficient value of
+    1 represents a galaxy image with all its light concentrated in just
+    one pixel.
 
     Usually Gini's measurement needs some sort of preprocessing for
     defining the galaxy region in the image based on the quality of the
@@ -46,9 +50,8 @@ def gini(data):
     """
 
     flattened = np.sort(np.ravel(data))
-    N = np.size(flattened)
-    normalization = 1. / (np.abs(np.mean(flattened)) * N * (N - 1))
-    kernel = (2 * np.arange(1, N + 1) - N - 1) * np.abs(flattened)
-    G = normalization * np.sum(kernel)
+    npix = np.size(flattened)
+    normalization = np.abs(np.mean(flattened)) * npix * (npix - 1)
+    kernel = (2. * np.arange(1, npix + 1) - npix - 1) * np.abs(flattened)
 
-    return G
+    return np.sum(kernel) / normalization
