@@ -1,24 +1,24 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 """
-Tools to build and fit an effective PSF (ePSF) based on Anderson and
-King (2000; PASP 112, 1360).
+This module provides tools to build and fit an effective PSF (ePSF)
+based on Anderson and King (2000; PASP 112, 1360) and Anderson (2016),
+ISR WFC3 2016-12.
 """
 
 import copy
 import time
 import warnings
 
-import numpy as np
 from astropy.modeling.fitting import LevMarLSQFitter
 from astropy.nddata.utils import (overlap_slices, PartialOverlapError,
                                   NoOverlapError)
-from astropy.utils.exceptions import (AstropyUserWarning,
-                                      AstropyDeprecationWarning)
+from astropy.utils.exceptions import AstropyUserWarning
+import numpy as np
 
-from .epsf_stars import EPSFStar, LinkedEPSFStar, EPSFStars
+from .epsf_stars import EPSFStar, EPSFStars, LinkedEPSFStar
 from .models import EPSFModel
-from ..centroids import centroid_com
-from ..extern import SigmaClip
+from ..centroids import centroid_epsf
+from ..extern.sigma_clipping import SigmaClip
 
 try:
     import bottleneck  # pylint: disable=W0611
@@ -26,7 +26,6 @@ try:
 except ImportError:
     HAS_BOTTLENECK = False
 
-
 __all__ = ['EPSFFitter', 'EPSFBuilder']
 
 
@@ -37,8 +36,7 @@ class EPSFFitter:
     Parameters
     ----------
     fitter : `astropy.modeling.fitting.Fitter`, optional
-        A `~astropy.modeling.fitting.Fitter` object.  The default is
-        `~astropy.modeling.fitting.LevMarLSQFitter`.
+        A `~astropy.modeling.fitting.Fitter` object.
 
     fit_boxsize : int, tuple of int, or `None`, optional
         The size (in pixels) of the box centered on the star to be used
@@ -48,8 +46,7 @@ class EPSFFitter:
         of size ``fit_boxsize`` will be used.  If ``fit_boxsize`` has
         two elements, they should be in ``(ny, nx)`` order.  The size
         must be greater than or equal to 3 pixels for both axes.  If
-        `None`, the fitter will use the entire star image.  The default
-        is 5.
+        `None`, the fitter will use the entire star image.
 
     fitter_kwargs : dict-like, optional
         Any additional keyword arguments (except ``x``, ``y``, ``z``, or
@@ -157,10 +154,9 @@ class EPSFFitter:
         if fit_boxsize is not None:
             try:
                 xcenter, ycenter = star.cutout_center
-                large_slc, small_slc = overlap_slices(star.shape,
-                                                      fit_boxsize,
-                                                      (ycenter, xcenter),
-                                                      mode='strict')
+                large_slc, _ = overlap_slices(star.shape, fit_boxsize,
+                                              (ycenter, xcenter),
+                                              mode='strict')
             except (PartialOverlapError, NoOverlapError):
                 warnings.warn('The star at ({0}, {1}) cannot be fit because '
                               'its fitting region extends beyond the star '
@@ -188,34 +184,24 @@ class EPSFFitter:
             x0 = 0
             y0 = 0
 
-        x_oversamp = star.pixel_scale[0] / epsf.pixel_scale[0]
-        y_oversamp = star.pixel_scale[1] / epsf.pixel_scale[1]
-        scaled_data = data / (x_oversamp * y_oversamp)
-
-        # define positions in the ePSF oversampled grid
+        # Define positions in the undersampled grid. The fitter will
+        # evaluate on the defined interpolation grid, currently in the
+        # range [0, len(undersampled grid)].
         yy, xx = np.indices(data.shape, dtype=np.float)
-        xx = (xx - (star.cutout_center[0] - x0)) * x_oversamp
-        yy = (yy - (star.cutout_center[1] - y0)) * y_oversamp
+        xx = xx + x0 - star.cutout_center[0]
+        yy = yy + y0 - star.cutout_center[1]
 
         # define the initial guesses for fitted flux and shifts
         epsf.flux = star.flux
         epsf.x_0 = 0.0
         epsf.y_0 = 0.0
 
-        # The oversampling factor is used in the FittableImageModel
-        # evaluate method (which is use when fitting).  We do not want
-        # to use oversampling here because it has been set by the ratio
-        # of the ePSF and EPSFStar pixel scales.  This allows for
-        # oversampling factors that differ between stars and also for
-        # the factor to be different along the x and y axes.
-        epsf._oversampling = 1.
-
         try:
-            fitted_epsf = fitter(model=epsf, x=xx, y=yy, z=scaled_data,
+            fitted_epsf = fitter(model=epsf, x=xx, y=yy, z=data,
                                  weights=weights, **fitter_kwargs)
         except TypeError:
             # fitter doesn't support weights
-            fitted_epsf = fitter(model=epsf, x=xx, y=yy, z=scaled_data,
+            fitted_epsf = fitter(model=epsf, x=xx, y=yy, z=data,
                                  **fitter_kwargs)
 
         fit_error_status = 0
@@ -223,15 +209,13 @@ class EPSFFitter:
             fit_info = copy.copy(fitter.fit_info)
 
             if 'ierr' in fit_info and fit_info['ierr'] not in [1, 2, 3, 4]:
-                fit_error_status = 2    # fit solution was not found
+                fit_error_status = 2  # fit solution was not found
         else:
             fit_info = None
 
         # compute the star's fitted position
-        x_center = (star.cutout_center[0] +
-                    (fitted_epsf.x_0.value / x_oversamp))
-        y_center = (star.cutout_center[1] +
-                    (fitted_epsf.y_0.value / y_oversamp))
+        x_center = star.cutout_center[0] + fitted_epsf.x_0.value
+        y_center = star.cutout_center[1] + fitted_epsf.y_0.value
 
         star = copy.deepcopy(star)
         star.cutout_center = (x_center, y_center)
@@ -250,39 +234,19 @@ class EPSFBuilder:
     Class to build an effective PSF (ePSF).
 
     See `Anderson and King (2000; PASP 112, 1360)
-    <http://adsabs.harvard.edu/abs/2000PASP..112.1360A>`_ for details.
+    <http://adsabs.harvard.edu/abs/2000PASP..112.1360A>`_ and
+    `Anderson (2016), ISR WFC3 2016-12
+    <www.stsci.edu/hst/wfc3/documents/ISRs/WFC3-2016-12.pdf>`_ for details.
 
     Parameters
     ----------
-    pixel_scale : float or tuple of two floats, optional
-        .. warning::
-
-            The ``pixel_scale`` keyword is now deprecated (since v0.6)
-            and will likely be removed in v0.7.  Use the
-            ``oversampling`` keyword instead.
-
-        The pixel scale (in arbitrary units) of the output ePSF.  The
-        ``pixel_scale`` can either be a single float or tuple of two
-        floats of the form ``(x_pixscale, y_pixscale)``.  If
-        ``pixel_scale`` is a scalar then the pixel scale will be the
-        same for both the x and y axes.  The ePSF ``pixel_scale`` is
-        used in conjunction with the star pixel scale when building and
-        fitting the ePSF.  This allows for building (and fitting) a ePSF
-        using images of stars with different pixel scales (e.g. velocity
-        aberrations).  Either ``oversampling`` or ``pixel_scale`` must
-        be input.  If both are input, ``pixel_scale`` will override the
-        input ``oversampling``.
-
-    oversampling : float or tuple of two floats, optional
+    oversampling : int or tuple of two int, optional
         The oversampling factor(s) of the ePSF relative to the input
-        ``stars`` along the x and y axes.  The ``oversampling`` can
+        ``stars`` along the x and y axes. The ``oversampling`` can
         either be a single float or a tuple of two floats of the form
         ``(x_oversamp, y_oversamp)``.  If ``oversampling`` is a scalar
         then the oversampling will be the same for both the x and y
-        axes.  The ``oversampling`` factor will be used with the minimum
-        pixel scale of all input stars to calculate the ePSF pixel
-        scale.  Either ``oversampling`` or ``pixel_scale`` must be
-        input.  If both are input, ``oversampling`` will be ignored.
+        axes.
 
     shape : float, tuple of two floats, or `None`, optional
         The shape of the output ePSF.  If the ``shape`` is not `None`,
@@ -297,77 +261,79 @@ class EPSFBuilder:
         ``'quartic'`` and ``'quadratic'`` kernels are derived from
         fourth and second degree polynomials, respectively.
         Alternatively, a custom 2D array can be input.  If `None` then
-        no smoothing will be performed.  The default is ``'quartic'``.
+        no smoothing will be performed.
 
     recentering_func : callable, optional
         A callable object (e.g. function or class) that is used to
         calculate the centroid of a 2D array.  The callable must accept
-        a 2D `~numpy.ndarray`, have a ``mask`` keyword and optionally an
-        ``error`` keyword.  The callable object must return a tuple of
-        two 1D `~numpy.ndarray`\\s, representing the x and y centroids.
-        The default is `~photutils.centroids.centroid_com`.
-
-    recentering_boxsize : float or tuple of two floats, optional
-        The size (in pixels) of the box used to calculate the centroid
-        of the ePSF during each build iteration.  If a single integer
-        number is provided, then a square box will be used.  If two
-        values are provided, then they should be in ``(ny, nx)`` order.
-        The default is 5.
+        a 2D `~numpy.ndarray`, have a ``mask`` keyword and optionally
+        ``error`` and ``oversampling`` keywords.  The callable object
+        must return a tuple of two 1D `~numpy.ndarray` variables,
+        representing the x and y centroids.
 
     recentering_maxiters : int, optional
         The maximum number of recentering iterations to perform during
-        each ePSF build iteration.  The default is 20.
+        each ePSF build iteration.
 
     fitter : `EPSFFitter` object, optional
-        A `EPSFFitter` object use to fit the ePSF to stars.  The default
-        fitter used by `EPSFFitter` is
-        `~astropy.modeling.fitting.LevMarLSQFitter`.  See the
-        `EPSFFitter` documentation its options.
-
-    center_accuracy : float, optional
-        The desired accuracy for the centers of stars.  The building
-        iterations will stop if the centers of all the stars change by
-        less than ``center_accuracy`` pixels between iterations.  All
-        stars must meet this condition for the loop to exit.  The
-        default is 1.0e-3.
+        A `EPSFFitter` object use to fit the ePSF to stars.  To set
+        fitter options, a new object with specific options should be
+        passed in - the default uses simply the default options.  To see
+        more of these options, see the `EPSFFitter` documentation.
 
     maxiters : int, optional
-        The maximum number of iterations to perform.  If the
-        ``center_accuracy`` is met, then the iterations will stop prior
-        to ``maxiters``.  The default is 10.
+        The maximum number of iterations to perform.
 
     progress_bar : bool, option
         Whether to print the progress bar during the build iterations.
-        The default is `True`.
+
+    norm_radius : float, optional
+        The pixel radius over which the ePSF is normalized.
+
+    shift_val : float, optional
+        The undersampled value at which to compute the shifts.  It must
+        be a strictly positive number.
+
+    recentering_boxsize : float or tuple of two floats, optional
+            The size (in pixels) of the box used to calculate the centroid
+            of the ePSF during each build iteration.  If a single integer
+            number is provided, then a square box will be used.  If two
+            values are provided, then they should be in ``(ny, nx)`` order.
+
+    center_accuracy : float, optional
+            The desired accuracy for the centers of stars.  The building
+            iterations will stop if the centers of all the stars change by
+            less than ``center_accuracy`` pixels between iterations.  All
+            stars must meet this condition for the loop to exit.
     """
 
-    def __init__(self, pixel_scale=None, oversampling=4., shape=None,
-                 smoothing_kernel='quartic', recentering_func=centroid_com,
-                 recentering_boxsize=(5, 5), recentering_maxiters=20,
-                 fitter=EPSFFitter(), center_accuracy=1.0e-3, maxiters=10,
-                 progress_bar=True):
+    def __init__(self, oversampling=4., shape=None,
+                 smoothing_kernel='quartic', recentering_func=centroid_epsf,
+                 recentering_maxiters=20, fitter=EPSFFitter(), maxiters=10,
+                 progress_bar=True, norm_radius=5.5, shift_val=0.5,
+                 recentering_boxsize=(5, 5), center_accuracy=1.0e-3):
+        flux_residual_sigclip = SigmaClip(sigma=3, cenfunc='median', maxiters=10)  # TODO: replace with kwarg in 8.0
 
-        if pixel_scale is None and oversampling is None:
-            raise ValueError('Either pixel_scale or oversampling must be '
-                             'input.')
-
-        if pixel_scale is not None:
-            warnings.warn('The pixel_scale keyword is deprecated and will '
-                          'likely be removed in v0.7.  Use the oversampling '
-                          'keyword instead.', AstropyDeprecationWarning)
-
-        self.pixel_scale = self._init_img_params(pixel_scale)
-        if oversampling <= 0.0:
+        if oversampling is None:
+            raise ValueError("'oversampling' must be specified.")
+        oversampling = np.atleast_1d(oversampling).astype(int)
+        if len(oversampling) == 1:
+            oversampling = np.repeat(oversampling, 2)
+        if np.any(oversampling % 2 != 0):
+            raise ValueError('Oversampling factor must be a multiple of two.')
+        if np.any(oversampling <= 0.0):
             raise ValueError('oversampling must be a positive number.')
+        self._norm_radius = norm_radius
+        self._shift_val = shift_val
         self.oversampling = oversampling
         self.shape = self._init_img_params(shape)
         if self.shape is not None:
             self.shape = self.shape.astype(int)
 
         self.recentering_func = recentering_func
+        self.recentering_maxiters = recentering_maxiters
         self.recentering_boxsize = self._init_img_params(recentering_boxsize)
         self.recentering_boxsize = self.recentering_boxsize.astype(int)
-        self.recentering_maxiters = recentering_maxiters
 
         self.smoothing_kernel = smoothing_kernel
 
@@ -381,23 +347,18 @@ class EPSFBuilder:
 
         maxiters = int(maxiters)
         if maxiters <= 0:
-            raise ValueError('maxiters must be a positive number.')
+            raise ValueError("'maxiters' must be a positive number.")
         self.maxiters = maxiters
 
         self.progress_bar = progress_bar
 
-        # TODO: allow custom SigmaClip object after faster SigmaClip
-        # is available in astropy (>=3.1)
-        self.sigclip = SigmaClip(sigma=3., cenfunc='median', maxiters=10)
+        if not isinstance(flux_residual_sigclip, SigmaClip):
+            raise ValueError("'flux_residual_sigclip' must be an"
+                             " astropy.stats.SigmaClip function.")
+        self.flux_residual_sigclip = flux_residual_sigclip
 
-        # store some data during each ePSF build iteration
-        self._nfit_failed = []
-        self._center_dist_sq = []
-        self._max_center_dist_sq = []
+        # store each ePSF build iteration
         self._epsf = []
-        self._residuals = []
-        self._residuals_sigclip = []
-        self._residuals_interp = []
 
     def __call__(self, stars):
         return self.build_epsf(stars)
@@ -420,9 +381,7 @@ class EPSFBuilder:
         """
         Create an initial `EPSFModel` object.
 
-        The initial ePSF data are all zeros.  The ePSF pixel scale is
-        determined either from the ``pixel_scale`` or ``oversampling``
-        values.
+        The initial ePSF data are all zeros.
 
         If ``shape`` is not specified, the shape of the ePSF data array
         is determined from the shape of the input ``stars`` and the
@@ -441,55 +400,42 @@ class EPSFBuilder:
             The initial ePSF model.
         """
 
-        pixel_scale = self.pixel_scale
+        norm_radius = self._norm_radius
+        shift_val = self._shift_val
         oversampling = self.oversampling
         shape = self.shape
 
-        if pixel_scale is None and oversampling is None:
-            raise ValueError('Either pixel_scale or oversampling must be '
-                             'input.')
-
-        # define the ePSF pixel scale
-        if pixel_scale is not None:
-            pixel_scale = np.atleast_1d(pixel_scale).astype(float)
-            if len(pixel_scale) == 1:
-                pixel_scale = np.repeat(pixel_scale, 2)
-
-            oversampling = (stars._min_pixel_scale[0] / pixel_scale[0],
-                            stars._min_pixel_scale[1] / pixel_scale[1])
-        else:
-            oversampling = np.atleast_1d(oversampling).astype(float)
-            if len(oversampling) == 1:
-                oversampling = np.repeat(oversampling, 2)
-
-            pixel_scale = (stars._min_pixel_scale[0] / oversampling[0],
-                           stars._min_pixel_scale[1] / oversampling[1])
-
         # define the ePSF shape
         if shape is not None:
             shape = np.atleast_1d(shape).astype(int)
             if len(shape) == 1:
                 shape = np.repeat(shape, 2)
         else:
-            x_shape = np.int(np.ceil(stars._max_shape[0] *
-                                     oversampling[1]))
-            y_shape = np.int(np.ceil(stars._max_shape[1] *
-                                     oversampling[0]))
+            # Stars class should have odd-sized dimensions, and thus we
+            # get the oversampled shape as oversampling * len + 1; if
+            # len=25, then newlen=101, for example.
+            x_shape = np.int(np.ceil(stars._max_shape[0]) * oversampling[0]
+                             + 1)
+            y_shape = np.int(np.ceil(stars._max_shape[1]) * oversampling[1]
+                             + 1)
             shape = np.array((y_shape, x_shape))
 
-        # ensure odd sizes
+        # verify odd sizes of shape
         shape = [(i + 1) if i % 2 == 0 else i for i in shape]
 
         data = np.zeros(shape, dtype=np.float)
-        xcenter = (shape[1] - 1) / 2.
-        ycenter = (shape[0] - 1) / 2.
 
-        # FittableImageModel requires a scalar oversampling factor
-        oversampling = np.mean(oversampling)
+        # ePSF origin should be in the undersampled pixel units, not the
+        # oversampled grid units. The middle, fractional (as we wish for
+        # the center of the pixel, so the center should be at (v.5, w.5)
+        # detector pixels) value is simply the average of the two values
+        # at the extremes.
+        xcenter = stars._max_shape[0] / 2.
+        ycenter = stars._max_shape[1] / 2.
 
         epsf = EPSFModel(data=data, origin=(xcenter, ycenter),
-                         normalize=False, oversampling=oversampling)
-        epsf._pixel_scale = pixel_scale
+                         oversampling=oversampling, norm_radius=norm_radius,
+                         shift_val=shift_val)
 
         return epsf
 
@@ -519,34 +465,36 @@ class EPSFBuilder:
             image contains NaNs where there is no data.
         """
 
-        # find the integer index of EPSFStar pixels in the oversampled
-        # ePSF grid
-        x_oversamp = star.pixel_scale[0] / epsf.pixel_scale[0]
-        y_oversamp = star.pixel_scale[1] / epsf.pixel_scale[1]
-        x = x_oversamp * star._xidx_centered
-        y = y_oversamp * star._yidx_centered
-        epsf_xcenter, epsf_ycenter = epsf.origin
+        # Compute the normalized residual by subtracting the ePSF model
+        # from the normalized star at the location of the star in the
+        # undersampled grid.
+
+        x = star._xidx_centered
+        y = star._yidx_centered
+
+        stardata = (star._data_values_normalized -
+                    epsf.evaluate(x=x, y=y, flux=1.0, x_0=0.0, y_0=0.0))
+
+        x = epsf.oversampling[0] * star._xidx_centered
+        y = epsf.oversampling[1] * star._yidx_centered
+
+        epsf_xcenter, epsf_ycenter = (int((epsf.data.shape[1] -
+                                           1) / 2),
+                                      int((epsf.data.shape[0] -
+                                           1) / 2))
         xidx = _py2intround(x + epsf_xcenter)
         yidx = _py2intround(y + epsf_ycenter)
 
-        mask = np.logical_and(np.logical_and(xidx >= 0, xidx < epsf.shape[1]),
-                              np.logical_and(yidx >= 0, yidx < epsf.shape[0]))
-        xidx = xidx[mask]
-        yidx = yidx[mask]
-
-        # Compute the normalized residual image by subtracting the
-        # normalized ePSF model from the normalized star at the location
-        # of the star in the undersampled grid.  Then, resample the
-        # normalized residual image in the oversampled ePSF grid.
-        # [(star - (epsf * xov * yov)) / (xov * yov)]
-        # --> [(star / (xov * yov)) - epsf]
-        stardata = ((star._data_values_normalized /
-                     (x_oversamp * y_oversamp)) -
-                    epsf.evaluate(x=x, y=y, flux=1.0, x_0=0.0, y_0=0.0,
-                                  use_oversampling=False))
-
         resampled_img = np.full(epsf.shape, np.nan)
-        resampled_img[yidx, xidx] = stardata[mask]
+
+        mask = np.logical_and(np.logical_and(xidx >= 0,
+                                             xidx < epsf.shape[1]),
+                              np.logical_and(yidx >= 0,
+                                             yidx < epsf.shape[0]))
+        xidx_ = xidx[mask]
+        yidx_ = yidx[mask]
+
+        resampled_img[yidx_, xidx_] = stardata[mask]
 
         return resampled_img
 
@@ -564,16 +512,16 @@ class EPSFBuilder:
 
         Returns
         -------
-        star_imgs : 3D `~numpy.ndarray`
+        epsf_resid : 3D `~numpy.ndarray`
             A 3D cube containing the resampled residual images.
         """
 
         shape = (stars.n_good_stars, epsf.shape[0], epsf.shape[1])
-        star_imgs = np.zeros(shape)
+        epsf_resid = np.zeros(shape)
         for i, star in enumerate(stars.all_good_stars):
-            star_imgs[i, :, :] = self._resample_residual(star, epsf)
+            epsf_resid[i, :, :] = self._resample_residual(star, epsf)
 
-        return star_imgs
+        return epsf_resid
 
     def _smooth_epsf(self, epsf_data):
         """
@@ -630,24 +578,21 @@ class EPSFBuilder:
                   -0.07428311]])
 
         elif isinstance(self.smoothing_kernel, np.ndarray):
-            kernel = self.kernel
+            kernel = self.smoothing_kernel
 
         else:
             raise TypeError('Unsupported kernel.')
 
         return convolve(epsf_data, kernel)
 
-    def _recenter_epsf(self, epsf_data, epsf, centroid_func=centroid_com,
-                       box_size=5, maxiters=20, center_accuracy=1.0e-4):
+    def _recenter_epsf(self, epsf, centroid_func=centroid_epsf,
+                       box_size=(5, 5), maxiters=20, center_accuracy=1.0e-4):
         """
         Calculate the center of the ePSF data and shift the data so the
         ePSF center is at the center of the ePSF data array.
 
         Parameters
         ----------
-        epsf_data : 2D `~numpy.ndarray`
-            A 2D array containing the ePSF image.
-
         epsf : `EPSFModel` object
             The ePSF model.
 
@@ -656,26 +601,20 @@ class EPSFBuilder:
             calculate the centroid of a 2D array.  The callable must
             accept a 2D `~numpy.ndarray`, have a ``mask`` keyword and
             optionally an ``error`` keyword.  The callable object must
-            return a tuple of two 1D `~numpy.ndarray`\\s, representing
-            the x and y centroids.  The default is
-            `~photutils.centroids.centroid_com`.
-
-        recentering_boxsize : float or tuple of two floats, optional
+            return a tuple of two 1D `~numpy.ndarray` variables, representing
+            the x and y centroids.
+        box_size : float or tuple of two floats, optional
             The size (in pixels) of the box used to calculate the
             centroid of the ePSF during each build iteration.  If a
             single integer number is provided, then a square box will be
             used.  If two values are provided, then they should be in
-            ``(ny, nx)`` order.  The default is 5.
-
+            ``(ny, nx)`` order.
         maxiters : int, optional
             The maximum number of recentering iterations to perform.
-            The default is 20.
-
         center_accuracy : float, optional
             The desired accuracy for the centers of stars.  The building
             iterations will stop if the center of the ePSF changes by
             less than ``center_accuracy`` pixels between iterations.
-            The default is 1.0e-4.
 
         Returns
         -------
@@ -683,59 +622,81 @@ class EPSFBuilder:
             The recentered ePSF data.
         """
 
-        # Define an EPSFModel for the input data.  This EPSFModel will be
-        # used to evaluate the model on a shifted pixel grid to place the
-        # centroid at the array center.
-        pixel_scale = epsf.pixel_scale
-        epsf = EPSFModel(data=epsf_data, origin=epsf.origin, normalize=False,
-                         oversampling=epsf.oversampling)
-        epsf._pixel_scale = pixel_scale
+        epsf_data = epsf._data
+
+        epsf = EPSFModel(data=epsf._data, origin=epsf.origin,
+                         oversampling=epsf.oversampling,
+                         norm_radius=epsf._norm_radius,
+                         shift_val=epsf._shift_val, normalize=False)
 
-        epsf.fill_value = 0.0
         xcenter, ycenter = epsf.origin
 
-        dx_total = 0
-        dy_total = 0
-        y, x = np.indices(epsf_data.shape, dtype=np.float)
+        y, x = np.indices(epsf._data.shape, dtype=np.float)
+        x /= epsf.oversampling[0]
+        y /= epsf.oversampling[1]
 
+        dx_total, dy_total = 0, 0
         iter_num = 0
         center_accuracy_sq = center_accuracy ** 2
         center_dist_sq = center_accuracy_sq + 1.e6
         center_dist_sq_prev = center_dist_sq + 1
         while (iter_num < maxiters and
                center_dist_sq >= center_accuracy_sq):
-
             iter_num += 1
 
-            # extract a cutout from the ePSF
-            slices_large, slices_small = overlap_slices(epsf_data.shape,
-                                                        box_size,
-                                                        (ycenter, xcenter))
-            epsf_cutout = epsf_data[slices_large]
+            # Anderson & King (2000) recentering function depends
+            # on specific pixels, and thus does not need a cutout
+            if self.recentering_func == centroid_epsf:
+                epsf_cutout = epsf_data
+            else:
+                slices_large, _ = overlap_slices(epsf_data.shape, box_size,
+                                                 (ycenter *
+                                                  self.oversampling[1],
+                                                  xcenter *
+                                                  self.oversampling[0]))
+                epsf_cutout = epsf_data[slices_large]
             mask = ~np.isfinite(epsf_cutout)
 
-            # find a new center position
-            xcenter_new, ycenter_new = centroid_func(epsf_cutout, mask=mask)
-            xcenter_new += slices_large[1].start
-            ycenter_new += slices_large[0].start
+            try:
+                # find a new center position
+                xcenter_new, ycenter_new = centroid_func(
+                    epsf_cutout, mask=mask, oversampling=epsf.oversampling,
+                    shift_val=epsf._shift_val)
+            except TypeError:
+                # centroid_func doesn't accept oversampling and/or shift_val
+                # keywords - try oversampling alone
+                try:
+                    xcenter_new, ycenter_new = centroid_func(
+                        epsf_cutout, mask=mask, oversampling=epsf.oversampling)
+                except TypeError:
+                    # centroid_func doesn't accept oversampling and
+                    # shift_val
+                    xcenter_new, ycenter_new = centroid_func(epsf_cutout,
+                                                             mask=mask)
+
+            if self.recentering_func != centroid_epsf:
+                xcenter_new += slices_large[1].start/self.oversampling[0]
+                ycenter_new += slices_large[0].start/self.oversampling[1]
+
+            # Calculate the shift; dx = i - x_star so if dx was positively
+            # incremented then x_star was negatively incremented for a given i.
+            # We will therefore actually subsequently subtract dx from xcenter
+            # (or x_star).
+            dx = xcenter_new - xcenter
+            dy = ycenter_new - ycenter
 
-            # calculate the shift
-            dx = xcenter - xcenter_new
-            dy = ycenter - ycenter_new
             center_dist_sq = dx**2 + dy**2
+
             if center_dist_sq >= center_dist_sq_prev:  # don't shift
                 break
             center_dist_sq_prev = center_dist_sq
 
-            # Resample the ePSF data to a shifted grid to place the
-            # centroid in the center of the central pixel.  The shift is
-            # always performed on the input epsf_data.
-            dx_total += dx    # accumulated shifts for the input epsf_data
+            dx_total += dx
             dy_total += dy
+
             epsf_data = epsf.evaluate(x=x, y=y, flux=1.0,
-                                      x_0=xcenter + dx_total,
-                                      y_0=ycenter + dy_total,
-                                      use_oversampling=False)
+                                      x_0=xcenter - dx_total,
+                                      y_0=ycenter - dy_total)
 
         return epsf_data
 
@@ -772,21 +733,18 @@ class EPSFBuilder:
         # compute a 3D stack of 2D residual images
         residuals = self._resample_residuals(stars, epsf)
 
-        self._residuals.append(residuals)
-
-        # compute the sigma-clipped median along the 3D stack
+        # compute the sigma-clipped average along the 3D stack
         with warnings.catch_warnings():
             warnings.simplefilter('ignore', category=RuntimeWarning)
             warnings.simplefilter('ignore', category=AstropyUserWarning)
-            residuals = self.sigclip(residuals, axis=0, masked=False,
-                                     return_bounds=False)
+            residuals = self.flux_residual_sigclip(residuals, axis=0,
+                                                   masked=False,
+                                                   return_bounds=False)
             if HAS_BOTTLENECK:
                 residuals = bottleneck.nanmedian(residuals, axis=0)
             else:
                 residuals = np.nanmedian(residuals, axis=0)
 
-        self._residuals_sigclip.append(residuals)
-
         # interpolate any missing data (np.nan)
         mask = ~np.isfinite(residuals)
         if np.any(mask):
@@ -796,33 +754,31 @@ class EPSFBuilder:
             # fill any remaining nans (outer points) with zeros
             residuals[~np.isfinite(residuals)] = 0.
 
-        self._residuals_interp.append(residuals)
-
         # add the residuals to the previous ePSF image
-        new_epsf = epsf.normalized_data + residuals
+        new_epsf = epsf._data + residuals
 
-        # smooth the ePSF
+        # smooth and recenter the ePSF
         new_epsf = self._smooth_epsf(new_epsf)
 
-        # recenter the ePSF
-        new_epsf = self._recenter_epsf(new_epsf, epsf,
-                                       centroid_func=self.recentering_func,
-                                       box_size=self.recentering_boxsize,
-                                       maxiters=self.recentering_maxiters,
-                                       center_accuracy=1.0e-4)
+        epsf = EPSFModel(data=new_epsf, origin=epsf.origin,
+                         oversampling=epsf.oversampling,
+                         norm_radius=epsf._norm_radius,
+                         shift_val=epsf._shift_val, normalize=False)
 
-        # normalize the ePSF data
-        new_epsf /= np.sum(new_epsf, dtype=np.float64)
+        epsf._data = self._recenter_epsf(
+            epsf, centroid_func=self.recentering_func,
+            box_size=self.recentering_boxsize,
+            maxiters=self.recentering_maxiters)
 
-        # return the new ePSF object
-        xcenter = (new_epsf.shape[1] - 1) / 2.
-        ycenter = (new_epsf.shape[0] - 1) / 2.
+        # Return the new ePSF object, but with undersampled grid pixel
+        # coordinates.
+        xcenter = (epsf._data.shape[1] - 1) / 2. / epsf.oversampling[0]
+        ycenter = (epsf._data.shape[0] - 1) / 2. / epsf.oversampling[1]
 
-        epsf_new = EPSFModel(data=new_epsf, origin=(xcenter, ycenter),
-                             normalize=False, oversampling=epsf.oversampling)
-        epsf_new._pixel_scale = epsf.pixel_scale
-
-        return epsf_new
+        return EPSFModel(data=epsf._data, origin=(xcenter, ycenter),
+                         oversampling=epsf.oversampling,
+                         norm_radius=epsf._norm_radius,
+                         shift_val=epsf._shift_val)
 
     def build_epsf(self, stars, init_epsf=None):
         """
@@ -848,17 +804,15 @@ class EPSFBuilder:
         """
 
         iter_num = 0
-        center_dist_sq = self.center_accuracy_sq + 1.
-        centers = stars.cutout_center_flat
         n_stars = stars.n_stars
         fit_failed = np.zeros(n_stars, dtype=bool)
-        dx_dy = np.zeros((n_stars, 2), dtype=np.float)
         epsf = init_epsf
         dt = 0.
+        center_dist_sq = self.center_accuracy_sq + 1.
+        centers = stars.cutout_center_flat
 
-        while (iter_num < self.maxiters and
-                np.max(center_dist_sq) >= self.center_accuracy_sq and
-                not np.all(fit_failed)):
+        while (iter_num < self.maxiters and not np.all(fit_failed) and
+               np.max(center_dist_sq) >= self.center_accuracy_sq):
 
             t_start = time.time()
             iter_num += 1
@@ -897,14 +851,13 @@ class EPSFBuilder:
                 for i in idx:
                     stars.all_stars[i]._excluded_from_fit = True
 
+            # if no star centers have moved by more than pixel accuracy,
+            # stop the iteration loop early
             dx_dy = stars.cutout_center_flat - centers
             dx_dy = dx_dy[np.logical_not(fit_failed)]
             center_dist_sq = np.sum(dx_dy * dx_dy, axis=1, dtype=np.float64)
             centers = stars.cutout_center_flat
 
-            self._nfit_failed.append(np.count_nonzero(fit_failed))
-            self._center_dist_sq.append(center_dist_sq)
-            self._max_center_dist_sq.append(np.max(center_dist_sq))
             self._epsf.append(epsf)
 
             dt = time.time() - t_start
@@ -964,10 +917,10 @@ def _interpolate_missing_data(data, mask, method='cubic'):
     data_interp = np.array(data, copy=True)
 
     if len(data_interp.shape) != 2:
-        raise ValueError('data must be a 2D array.')
+        raise ValueError("'data' must be a 2D array.")
 
     if mask.shape != data.shape:
-        raise ValueError('mask and data must have the same shape.')
+        raise ValueError("'mask' and 'data' must have the same shape.")
 
     y, x = np.indices(data_interp.shape)
     xy = np.dstack((x[~mask].ravel(), y[~mask].ravel()))[0]
