@@ -84,7 +84,7 @@ class DAOGroup(GroupStarsBase):
 
     See Also
     --------
-    photutils.DAOStarFinder
+    photutils.detection.DAOStarFinder
 
     References
     ----------
@@ -120,7 +120,7 @@ class DAOGroup(GroupStarsBase):
                                         data=np.arange(len(cstarlist)) + 1))
         cstarlist.add_column(Column(name='group_id',
                                     data=np.zeros(len(cstarlist),
-                                                  dtype=np.int)))
+                                                  dtype=int)))
 
         if not np.array_equal(cstarlist['id'], np.arange(len(cstarlist)) + 1):
             raise ValueError('id colum must be an integer-valued ' +
@@ -199,12 +199,12 @@ class DBSCANGroup(GroupStarsBase):
     References
     ----------
     [1] Scikit Learn DBSCAN.
-        http://scikit-learn.org/stable/modules/generated/sklearn.cluster.DBSCAN.html#sklearn.cluster.DBSCAN
+        https://scikit-learn.org/stable/modules/generated/sklearn.cluster.DBSCAN.html#sklearn.cluster.DBSCAN
 
     Notes
     -----
     * The attribute ``crit_separation`` corresponds to ``eps`` in
-      `sklearn.cluster.DBSCAN <http://scikit-learn.org/stable/modules/generated/sklearn.cluster.DBSCAN.html#sklearn.cluster.DBSCAN>`_.
+      `sklearn.cluster.DBSCAN <https://scikit-learn.org/stable/modules/generated/sklearn.cluster.DBSCAN.html#sklearn.cluster.DBSCAN>`_.
 
     * This class provides more general algorithms than
       `photutils.psf.DAOGroup`.  More precisely,
@@ -241,5 +241,5 @@ class DBSCANGroup(GroupStarsBase):
                         min_samples=self.min_samples, metric=self.metric,
                         algorithm=self.algorithm, leaf_size=self.leaf_size)
         cstarlist['group_id'] = (dbscan.fit(pos_stars).labels_ +
-                                 np.ones(len(cstarlist), dtype=np.int))
+                                 np.ones(len(cstarlist), dtype=int))
         return cstarlist
