@@ -1,18 +1,16 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 """
-Models for doing PSF/PRF fitting photometry on image data.
+This module provides models for doing PSF/PRF-fitting photometry.
 """
 
 import copy
 import itertools
 import warnings
 
-import numpy as np
+from astropy.modeling import Fittable2DModel, Parameter
 from astropy.nddata import NDData
-from astropy.modeling import Parameter, Fittable2DModel
-from astropy.utils.exceptions import (AstropyWarning,
-                                      AstropyDeprecationWarning)
-
+from astropy.utils.exceptions import AstropyWarning
+import numpy as np
 
 __all__ = ['NonNormalizable', 'FittableImageModel', 'EPSFModel',
            'GriddedPSFModel', 'IntegratedGaussianPRF', 'PRFAdapter']
@@ -22,9 +20,7 @@ class NonNormalizable(AstropyWarning):
     """
     Used to indicate that a :py:class:`FittableImageModel` model is
     non-normalizable.
-
     """
-    pass
 
 
 class FittableImageModel(Fittable2DModel):
@@ -32,19 +28,20 @@ class FittableImageModel(Fittable2DModel):
     A fittable 2D model of an image allowing for image intensity scaling
     and image translations.
 
-    This class takes 2D image data and computes the
-    values of the model at arbitrary locations (including at intra-pixel,
-    fractional positions) within this image using spline interpolation
-    provided by :py:class:`~scipy.interpolate.RectBivariateSpline`.
+    This class takes 2D image data and computes the values of the model
+    at arbitrary locations (including at intra-pixel, fractional
+    positions) within this image using spline interpolation provided by
+    :py:class:`~scipy.interpolate.RectBivariateSpline`.
 
-    The fittable model provided by this class has three model parameters:
-    an image intensity scaling factor (`flux`) which is applied to
-    (normalized) image, and two positional parameters (`x_0` and `y_0`)
-    indicating the location of a feature in the coordinate grid on which
-    the model is to be evaluated.
+    The fittable model provided by this class has three model
+    parameters: an image intensity scaling factor (``flux``) which is
+    applied to (normalized) image, and two positional parameters
+    (``x_0`` and ``y_0``) indicating the location of a feature in the
+    coordinate grid on which the model is to be evaluated.
 
-    If this class is initialized with `flux` (intensity scaling factor)
-    set to `None`, then `flux` is going to be estimated as ``sum(data)``.
+    If this class is initialized with ``flux`` (intensity scaling
+    factor) set to `None`, then ``flux`` is going to be estimated as
+    ``sum(data)``.
 
     Parameters
     ----------
@@ -52,61 +49,70 @@ class FittableImageModel(Fittable2DModel):
         Array containing 2D image.
 
     origin : tuple, None, optional
-        A reference point in the input image ``data`` array. When origin is
-        `None`, origin will be set at the middle of the image array.
+        A reference point in the input image ``data`` array. When origin
+        is `None`, origin will be set at the middle of the image array.
 
-        If `origin` represents the location of a feature (e.g., the position
-        of an intensity peak) in the input ``data``, then model parameters
-        `x_0` and `y_0` show the location of this peak in an another target
-        image to which this model was fitted. Fundamentally, it is the
-        coordinate in the model's image data that should map to
-        coordinate (`x_0`, `y_0`) of the output coordinate system on which the
-        model is evaluated.
+        If ``origin`` represents the location of a feature (e.g., the
+        position of an intensity peak) in the input ``data``, then model
+        parameters ``x_0`` and ``y_0`` show the location of this peak in
+        an another target image to which this model was fitted.
+        Fundamentally, it is the coordinate in the model's image data
+        that should map to coordinate (``x_0``, ``y_0``) of the output
+        coordinate system on which the model is evaluated.
 
-        Alternatively, when `origin` is set to ``(0,0)``, then model parameters
-        `x_0` and `y_0` are shifts by which model's image should be translated
-        in order to match a target image.
+        Alternatively, when ``origin`` is set to ``(0,0)``, then model
+        parameters ``x_0`` and ``y_0`` are shifts by which model's image
+        should be translated in order to match a target image.
 
     normalize : bool, optional
         Indicates whether or not the model should be build on normalized
-        input image data. If true, then the normalization constant (*N*) is
-        computed so that
+        input image data. If true, then the normalization constant (*N*)
+        is computed so that
 
         .. math::
-            N \\cdot C \\cdot \\Sigma_{i,j}D_{i,j} = 1,
+            N \\cdot C \\cdot \\sum\\limits_{i,j} D_{i,j} = 1,
 
-        where *N* is the normalization constant, *C* is correction factor
-        given by the parameter ``normalization_correction``, and
-        :math:`D_{i,j}` are the elements of the input image ``data`` array.
+        where *N* is the normalization constant, *C* is correction
+        factor given by the parameter ``normalization_correction``, and
+        :math:`D_{i,j}` are the elements of the input image ``data``
+        array.
 
     normalization_correction : float, optional
-        A strictly positive number that represents correction that needs to
-        be applied to model's data normalization (see *C* in the equation
-        in the comments to ``normalize`` for more details).
-
-        A possible application for this parameter is to account for aperture
-        correction. Assuming model's data represent a PSF to be fitted to
-        some target star, we set ``normalization_correction`` to the aperture
-        correction that needs to be applied to the model. That is,
-        ``normalization_correction`` in this case should be set to the
-        ratio between the total flux of the PSF (including flux outside model's
-        data) to the flux of model's data.
-        Then, best fitted value of the `flux` model
-        parameter will represent an aperture-corrected flux of the target star.
+        A strictly positive number that represents correction that needs
+        to be applied to model's data normalization (see *C* in the
+        equation in the comments to ``normalize`` for more details).  A
+        possible application for this parameter is to account for
+        aperture correction. Assuming model's data represent a PSF to be
+        fitted to some target star, we set ``normalization_correction``
+        to the aperture correction that needs to be applied to the
+        model. That is, ``normalization_correction`` in this case should
+        be set to the ratio between the total flux of the PSF (including
+        flux outside model's data) to the flux of model's data.  Then,
+        best fitted value of the ``flux`` model parameter will represent
+        an aperture-corrected flux of the target star.  In the case of
+        aperture correction, ``normalization_correction`` should be a
+        value larger than one, as the total flux, including regions
+        outside of the aperture, should be larger than the flux inside
+        the aperture, and thus the correction is applied as an inversely
+        multiplied factor.
 
     fill_value : float, optional
         The value to be returned by the `evaluate` or
-        ``astropy.modeling.Model.__call__`` methods
-        when evaluation is performed outside the definition domain of the
-        model.
+        ``astropy.modeling.Model.__call__`` methods when evaluation is
+        performed outside the definition domain of the model.
 
-    ikwargs : dict, optional
-
-        Additional optional keyword arguments to be passed directly to the
-        `compute_interpolator` method. See `compute_interpolator` for more
-        details.
+    kwargs : dict, optional
+        Additional optional keyword arguments to be passed directly to
+        the `compute_interpolator` method.  See `compute_interpolator`
+        for more details.
 
+    oversampling : float or tuple of two floats, optional
+        The oversampling factor(s) of the model in the ``x`` and ``y``
+        directions.  If ``oversampling`` is a scalar it will be treated
+        as being the same in both x and y; otherwise a tuple of two
+        floats will be treated as ``(x_oversamp, y_oversamp)``.
     """
+
     flux = Parameter(description='Intensity scaling factor for image data.',
                      default=1.0)
     x_0 = Parameter(description='X-position of a feature in the image in '
@@ -116,14 +122,15 @@ class FittableImageModel(Fittable2DModel):
                     'the output coordinate grid on which the model is '
                     'evaluated.', default=0.0)
 
-    def __init__(self, data, flux=flux.default,
-                 x_0=x_0.default, y_0=y_0.default,
-                 normalize=False, normalization_correction=1.0,
-                 origin=None, oversampling=1, fill_value=0.0, ikwargs={}):
+    def __init__(self, data, flux=flux.default, x_0=x_0.default,
+                 y_0=y_0.default, normalize=False,
+                 normalization_correction=1.0, origin=None, oversampling=1,
+                 fill_value=0.0, **kwargs):
+
         self._fill_value = fill_value
         self._img_norm = None
         self._normalization_status = 0 if normalize else 2
-        self._store_interpolator_kwargs(ikwargs)
+        self._store_interpolator_kwargs(**kwargs)
         self._set_oversampling(oversampling)
 
         if normalization_correction <= 0:
@@ -145,27 +152,34 @@ class FittableImageModel(Fittable2DModel):
         # set the origin of the coordinate system in image's pixel grid:
         self.origin = origin
 
-        if flux is None:
-            if self._img_norm is None:
-                self._img_norm = self._compute_raw_image_norm(self._data)
-            flux = self._img_norm
-
-        self._compute_normalization(normalize)
+        flux = self._initial_norm(flux, normalize)
 
         super().__init__(flux, x_0, y_0)
 
         # initialize interpolator:
-        self.compute_interpolator(ikwargs)
+        self.compute_interpolator(**kwargs)
 
-    def _compute_raw_image_norm(self, data):
+    def _initial_norm(self, flux, normalize):
+
+        if flux is None:
+            if self._img_norm is None:
+                self._img_norm = self._compute_raw_image_norm()
+            flux = self._img_norm
+
+        self._compute_normalization(normalize)
+
+        return flux
+
+    def _compute_raw_image_norm(self):
         """
-        Helper function that computes the uncorrected inverse normalization
-        factor of input image data. This quantity is computed as the
-        *sum of all pixel values*.
+        Helper function that computes the uncorrected inverse
+        normalization factor of input image data. This quantity is
+        computed as the *sum of all pixel values*.
 
         .. note::
-            This function is intended to be overriden in a subclass if one
-            desires to change the way the normalization factor is computed.
+            This function is intended to be overriden in a subclass if
+            one desires to change the way the normalization factor is
+            computed.
 
         """
         return np.sum(self._data, dtype=np.float64)
@@ -201,7 +215,7 @@ class FittableImageModel(Fittable2DModel):
             # compute normalization constant so that
             # N*C*sum(data) = 1:
             if self._img_norm is None:
-                self._img_norm = self._compute_raw_image_norm(self._data)
+                self._img_norm = self._compute_raw_image_norm()
 
             if self._img_norm != 0.0 and np.isfinite(self._img_norm):
                 self._normalization_constant /= self._img_norm
@@ -220,23 +234,28 @@ class FittableImageModel(Fittable2DModel):
     @property
     def oversampling(self):
         """
-        The factor by which the stored image is oversampled.  I.e., an input
-        to this model is multipled by this factor to yield the index into the
-        stored image.
+        The factor by which the stored image is oversampled.  I.e., an
+        input to this model is multipled by this factor to yield the
+        index into the stored image.
         """
+
         return self._oversampling
 
     def _set_oversampling(self, value):
         """
-        This is a private method because it's used in the initializer but the
-        ``oversampling``
+        This is a private method because it's used in the initializer by
+        the ``oversampling``
         """
+
         try:
-            value = float(value)
+            value = np.atleast_1d(value).astype(float)
+            if len(value) == 1:
+                value = np.repeat(value, 2)
         except ValueError:
-            raise ValueError('Oversampling factor must be a scalar')
-        if value <= 0:
-            raise ValueError('Oversampling factor must be greater than 0')
+            raise ValueError('Oversampling factors must be float')
+
+        if np.any(value <= 0):
+            raise ValueError('Oversampling factors must be greater than 0')
 
         self._oversampling = value
 
@@ -247,8 +266,8 @@ class FittableImageModel(Fittable2DModel):
 
     @property
     def normalized_data(self):
-        """ Get normalized and/or intensity-corrected image data. """
-        return (self._normalization_constant * self._data)
+        """Get normalized and/or intensity-corrected image data."""
+        return self._normalization_constant * self._data
 
     @property
     def normalization_constant(self):
@@ -261,11 +280,11 @@ class FittableImageModel(Fittable2DModel):
         Get normalization status. Possible status values are:
 
         - 0: **Performed**. Model has been successfuly normalized at
-          user's request.
+             user's request.
         - 1: **Failed**. Attempt to normalize has failed.
         - 2: **NotRequested**. User did not request model to be normalized.
-
         """
+
         return self._normalization_status
 
     @property
@@ -274,12 +293,12 @@ class FittableImageModel(Fittable2DModel):
         Set/Get flux correction factor.
 
         .. note::
-            When setting correction factor, model's flux will be adjusted
-            accordingly such that if this model was a good fit to some target
-            image before, then it will remain a good fit after correction
-            factor change.
-
+            When setting correction factor, model's flux will be
+            adjusted accordingly such that if this model was a good fit
+            to some target image before, then it will remain a good fit
+            after correction factor change.
         """
+
         return self._normalization_correction
 
     @normalization_correction.setter
@@ -288,14 +307,17 @@ class FittableImageModel(Fittable2DModel):
         self._normalization_correction = normalization_correction
         self._compute_normalization(normalize=self._normalization_status != 2)
 
-        # adjust model's flux so that if this model was a good fit to some
-        # target image, then it will remain a good fit after correction factor
-        # change:
+        # adjust model's flux so that if this model was a good fit to
+        # some target image, then it will remain a good fit after
+        # correction factor change:
         self.flux *= normalization_correction / old_cf
 
     @property
     def shape(self):
-        """A tuple of dimensions of the data array in numpy style (ny, nx)."""
+        """
+        A tuple of dimensions of the data array in numpy style (ny, nx).
+        """
+
         return self._shape
 
     @property
@@ -311,18 +333,19 @@ class FittableImageModel(Fittable2DModel):
     @property
     def origin(self):
         """
-        A tuple of ``x`` and ``y`` coordinates of the origin of the coordinate
-        system in terms of pixels of model's image.
+        A tuple of ``x`` and ``y`` coordinates of the origin of the
+        coordinate system in terms of pixels of model's image.
 
-        When setting the coordinate system origin, a tuple of two `int` or
-        `float` may be used. If origin is set to `None`, the origin of the
-        coordinate system will be set to the middle of the data array
-        (``(npix-1)/2.0``).
+        When setting the coordinate system origin, a tuple of two `int`
+        or `float` may be used. If origin is set to `None`, the origin
+        of the coordinate system will be set to the middle of the data
+        array (``(npix-1)/2.0``).
 
         .. warning::
-            Modifying `origin` will not adjust (modify) model's parameters
-            `x_0` and `y_0`.
+            Modifying ``origin`` will not adjust (modify) model's
+            parameters ``x_0`` and ``y_0``.
         """
+
         return (self._x_origin, self._y_origin)
 
     @origin.setter
@@ -348,24 +371,26 @@ class FittableImageModel(Fittable2DModel):
 
     @property
     def fill_value(self):
-        """Fill value to be returned for coordinates outside of the domain of
-        definition of the interpolator. If ``fill_value`` is `None`, then
-        values outside of the domain of definition are the ones returned
-        by the interpolator.
-
         """
+        Fill value to be returned for coordinates outside of the domain
+        of definition of the interpolator. If ``fill_value`` is `None`,
+        then values outside of the domain of definition are the ones
+        returned by the interpolator.
+        """
+
         return self._fill_value
 
     @fill_value.setter
     def fill_value(self, fill_value):
         self._fill_value = fill_value
 
-    def _store_interpolator_kwargs(self, ikwargs):
+    def _store_interpolator_kwargs(self, **kwargs):
         """
         This function should be called in a subclass whenever model's
         interpolator is (re-)computed.
         """
-        self._interpolator_kwargs = copy.deepcopy(ikwargs)
+
+        self._interpolator_kwargs = copy.deepcopy(kwargs)
 
     @property
     def interpolator_kwargs(self):
@@ -373,49 +398,52 @@ class FittableImageModel(Fittable2DModel):
         Get current interpolator's arguments used when interpolator was
         created.
         """
+
         return self._interpolator_kwargs
 
-    def compute_interpolator(self, ikwargs={}):
+    def compute_interpolator(self, **kwargs):
         """
-        Compute/define the interpolating spline. This function can be overriden
-        in a subclass to define custom interpolators.
+        Compute/define the interpolating spline.
+
+        This function can be overriden in a subclass to define custom
+        interpolators.
 
         Parameters
         ----------
-        ikwargs : dict, optional
-
-            Additional optional keyword arguments. Possible values are:
+        kwargs : dict, optional
+            Additional optional keyword arguments:
 
             - **degree** : int, tuple, optional
-                Degree of the interpolating spline. A tuple can be used to
-                provide different degrees for the X- and Y-axes.
+                Degree of the interpolating spline. A tuple can be used
+                to provide different degrees for the X- and Y-axes.
                 Default value is degree=3.
 
             - **s** : float, optional
-                Non-negative smoothing factor. Default value s=0 corresponds to
-                interpolation.
-                See :py:class:`~scipy.interpolate.RectBivariateSpline` for more
-                details.
+                Non-negative smoothing factor. Default value s=0
+                corresponds to interpolation.  See
+                :py:class:`~scipy.interpolate.RectBivariateSpline` for
+                more details.
 
         Notes
         -----
-            * When subclassing :py:class:`FittableImageModel` for the
-              purpose of overriding :py:func:`compute_interpolator`,
-              the :py:func:`evaluate` may need to overriden as well depending
-              on the behavior of the new interpolator. In addition, for
-              improved future compatibility, make sure
-              that the overriding method stores keyword arguments ``ikwargs``
-              by calling ``_store_interpolator_kwargs`` method.
-
-            * Use caution when modifying interpolator's degree or smoothness in
-              a computationally intensive part of the code as it may decrease
-              code performance due to the need to recompute interpolator.
+        * When subclassing :py:class:`FittableImageModel` for the
+          purpose of overriding :py:func:`compute_interpolator`, the
+          :py:func:`evaluate` may need to overriden as well depending on
+          the behavior of the new interpolator. In addition, for
+          improved future compatibility, make sure that the overriding
+          method stores keyword arguments ``kwargs`` by calling
+          ``_store_interpolator_kwargs`` method.
 
+        * Use caution when modifying interpolator's degree or smoothness
+          in a computationally intensive part of the code as it may
+          decrease code performance due to the need to recompute
+          interpolator.
         """
+
         from scipy.interpolate import RectBivariateSpline
 
-        if 'degree' in ikwargs:
-            degree = ikwargs['degree']
+        if 'degree' in kwargs:
+            degree = kwargs['degree']
             if hasattr(degree, '__iter__') and len(degree) == 2:
                 degx = int(degree[0])
                 degy = int(degree[1])
@@ -429,10 +457,7 @@ class FittableImageModel(Fittable2DModel):
             degx = 3
             degy = 3
 
-        if 's' in ikwargs:
-            smoothness = ikwargs['s']
-        else:
-            smoothness = 0
+        smoothness = kwargs.get('s', 0)
 
         x = np.arange(self._nx, dtype=np.float)
         y = np.arange(self._ny, dtype=np.float)
@@ -440,7 +465,7 @@ class FittableImageModel(Fittable2DModel):
             x, y, self._data.T, kx=degx, ky=degy, s=smoothness
         )
 
-        self._store_interpolator_kwargs(ikwargs)
+        self._store_interpolator_kwargs(**kwargs)
 
     def evaluate(self, x, y, flux, x_0, y_0, use_oversampling=True):
         """
@@ -456,8 +481,8 @@ class FittableImageModel(Fittable2DModel):
         """
 
         if use_oversampling:
-            xi = self._oversampling * (np.asarray(x) - x_0)
-            yi = self._oversampling * (np.asarray(y) - y_0)
+            xi = self._oversampling[0] * (np.asarray(x) - x_0)
+            yi = self._oversampling[1] * (np.asarray(y) - y_0)
         else:
             xi = np.asarray(x) - x_0
             yi = np.asarray(y) - y_0
@@ -480,75 +505,248 @@ class FittableImageModel(Fittable2DModel):
 
 class EPSFModel(FittableImageModel):
     """
-    A subclass of `FittableImageModel`.
+    A class that models an effective PSF (ePSF).
+
+    While this class is a subclass of `FittableImageModel`, it is very
+    similar.  The primary differences/motivation are a few additional
+    parameters necessary specifically for ePSFs.
 
     Parameters
     ----------
-    pixel_scale : float, tuple of two floats or `None`, optional
-        .. warning::
+    oversampling : int or tuple of two int, optional
+        The oversampling factor(s) of the model in the ``x`` and ``y``
+        directions.  If ``oversampling`` is a scalar it will be treated
+        as being the same in both x and y; otherwise a tuple of two
+        floats will be treated as ``(x_oversamp, y_oversamp)``.
 
-            The ``pixel_scale`` keyword is now deprecated (since v0.6)
-            and will likely be removed in v0.7.  Use the
-            ``oversampling`` keyword instead.
+    norm_radius : float, optional
+        The radius inside which the ePSF is normalized by the sum over
+        undersampled integer pixel values inside a circular aperture.
 
-        The pixel scale (in arbitrary units) of the ePSF.  The
-        ``pixel_scale`` can either be a single float or tuple of two
-        floats of the form ``(x_pixscale, y_pixscale)``.  If
-        ``pixel_scale`` is a scalar then the pixel scale will be the
-        same for both the x and y axes.  The default is `None`, which
-        means it will be set to the inverse of the ``oversampling``
-        factor.
-
-        The ePSF ``pixel_scale`` is used only when building the ePSF and
-        when fitting the ePSF to `Star` objects with `EPSFFitter`.  In
-        those cases, the ``pixel_scale`` is used in conjunction with the
-        `Star` pixel scale when building and fitting the ePSF.  This
-        allows for building (and fitting) a ePSF using images of stars
-        with different pixel scales (e.g. velocity aberrations).  The
-        ``oversampling`` factor is ignored in these cases.
-
-        If you are not using `EPSFBuilder` or `EPSFFitter`, then you
-        must set the ``oversampling`` factor.  The ``pixel_scale`` will
-        be ignored.
+    shift_val : float, optional
+        The fractional undersampled pixel amount (equivalent to an
+        integer oversampled pixel value) at which to evaluate the
+        asymmetric ePSF centroid corrections.
     """
 
-    def __init__(self, data, flux=1.0, x_0=0, y_0=0, normalize=True,
-                 normalization_correction=1.0, origin=None, oversampling=1.,
-                 pixel_scale=None, fill_value=0., ikwargs={}):
+    def __init__(self, data, flux=1.0, x_0=0.0, y_0=0.0, normalize=True,
+                 normalization_correction=1.0, origin=None, oversampling=1,
+                 fill_value=0.0, norm_radius=5.5, shift_val=0.5, **kwargs):
 
-        if pixel_scale is None:
-            pixel_scale = 1. / oversampling
+        self._norm_radius = norm_radius
+        self._shift_val = shift_val
+
+        super().__init__(data=data, flux=flux, x_0=x_0, y_0=y_0,
+                         normalize=normalize,
+                         normalization_correction=normalization_correction,
+                         origin=origin, oversampling=oversampling,
+                         fill_value=fill_value, **kwargs)
+
+    def _initial_norm(self, flux, normalize):
+        if flux is None:
+            if self._img_norm is None:
+                self._img_norm = self._compute_raw_image_norm()
+            flux = self._img_norm
+
+        if normalize:
+            self._compute_normalization()
         else:
-            warnings.warn('The pixel_scale keyword is deprecated and will '
-                          'likely be removed in v0.7.  Use the oversampling '
-                          'keyword instead.', AstropyDeprecationWarning)
+            self._img_norm = self._compute_raw_image_norm()
 
-        super().__init__(
-            data=data, flux=flux, x_0=x_0, y_0=y_0, normalize=normalize,
-            normalization_correction=normalization_correction, origin=origin,
-            oversampling=oversampling, fill_value=fill_value, ikwargs=ikwargs)
+        return flux
 
-        self._pixel_scale = pixel_scale
-
-    @property
-    def pixel_scale(self):
+    def _compute_raw_image_norm(self):
         """
-        The ``(x, y)`` pixel scale (in arbitrary units) of the PSF.
+        Helper function that computes the normalization of input image
+        data.  This quantity is computed as the sum of all undersampled
+        integer pixel values within radius pixels of the center of the
+        ePSF.
         """
 
-        return self._pixel_scale
+        # First need the indices of each axis at the oversampled
+        # resolution; if oversampling = 4 then x = [0, 0.25, 0.5, 0.75, ...]
+        x = np.arange(self._nx, dtype=np.float64) / self.oversampling[0]
+        y = np.arange(self._ny, dtype=np.float64) / self.oversampling[1]
 
-    @pixel_scale.setter
-    def pixel_scale(self, pixel_scale):
-        if pixel_scale is not None:
-            pixel_scale = np.atleast_1d(pixel_scale)
-            if len(pixel_scale) == 1:
-                pixel_scale = np.repeat(pixel_scale, 2).astype(float)
-            elif len(pixel_scale) > 2:
-                raise ValueError('pixel_scale must be a scalar or tuple '
-                                 'of two floats.')
+        # Take indices where the undersampled grid is an integer --
+        # i.e., the actual undersampled grid -- and find the cut where
+        # sqrt(dx**2 + dy**2) <= radius
+        x_0, y_0 = int((self._nx - 1) / 2), int((self._ny - 1) / 2)
 
-        self._pixel_scale = pixel_scale
+        # However, as we are in units of the undersampled grid, we must
+        # convert to undersampled units by the same factor of oversampling
+        x_0 /= self.oversampling[0]
+        y_0 /= self.oversampling[1]
+
+        # When checking if the index is at the center of a pixel, we
+        # check such that the index number is half that of the
+        # oversampling -- if we oversample by a factor 4 then the middle
+        # pixel of the 0th large pixel is 2 ([0, 1, 2, 3, 4]). For this to
+        # work we require oversampling to be an even number; otherwise,
+        # the ``middle'' pixel will be halfway between two oversampled
+        # pixels.
+        over_index_middle = 1 / 2
+        cut = (((x.reshape(1, -1) - x_0)**2 + (y.reshape(-1, 1) - y_0)**2 <=
+                self._norm_radius**2)
+               & (x.reshape(1, -1) % 1.0 == over_index_middle)
+               & (y.reshape(-1, 1) % 1.0 == over_index_middle))
+
+        return np.sum(self._data[cut], dtype=np.float64)
+
+    def _compute_normalization(self):
+        """
+        Helper function that computes (corrected) normalization factor
+        of the original image data. For the ePSF this is defined as the
+        sum over the inner N (default=5.5) pixels of the non-oversampled
+        image. Will re-normalize the data to the value calculated.
+        """
+
+        if self._img_norm is None:
+            if np.sum(self._data) == 0:
+                self._img_norm = 1
+            else:
+                self._img_norm = self._compute_raw_image_norm()
+
+        if self._img_norm != 0.0 and np.isfinite(self._img_norm):
+            self._data /= (self._img_norm * self._normalization_correction)
+            self._normalization_status = 0
+        else:
+            self._normalization_status = 1
+            self._img_norm = 1
+            warnings.warn("Overflow encountered while computing "
+                          "normalization constant. Normalization "
+                          "constant will be set to 1.", NonNormalizable)
+
+    def _set_oversampling(self, value):
+        try:
+            value = np.atleast_1d(value).astype(int)
+            if len(value) == 1:
+                value = np.repeat(value, 2)
+            # We need oversampling to be a factor of 2 for ``middle of
+            # pixel'' in the undersampled regime to have a pixel placed at
+            # it in the oversampled regime.
+            if np.any(value % 2 != 0) and np.logical_not(np.all(value == 1)):
+                raise ValueError('Oversampling factor must be a multiple of '
+                                 'two')
+        except ValueError:
+            raise ValueError('Oversampling factor must be a scalar')
+        if np.any(value <= 0):
+            raise ValueError('Oversampling factor must be greater than 0')
+
+        self._oversampling = value
+
+    def normalized_data(self):
+        """
+        Overloaded dummy function that also returns self._data, as the
+        normalization occurs within _compute_normalization in EPSFModel,
+        and as such self._data will sum, accounting for
+        under/oversampled pixels, to 1/self._normalization_correction.
+        """
+
+        return self._data
+
+    @FittableImageModel.origin.setter
+    def origin(self, origin):
+        if origin is None:
+            self._x_origin = (self._nx - 1) / 2.0 / self.oversampling[0]
+            self._y_origin = (self._ny - 1) / 2.0 / self.oversampling[1]
+        elif (hasattr(origin, '__iter__') and len(origin) == 2):
+            self._x_origin, self._y_origin = origin
+        else:
+            raise TypeError("Parameter 'origin' must be either None or an "
+                            "iterable with two elements.")
+
+    def compute_interpolator(self, **kwargs):
+        """
+        Compute/define the interpolating spline.
+
+        This function can be overriden in a subclass to define custom
+        interpolators.
+
+        Parameters
+        ----------
+        kwargs : dict, optional
+            Additional optional keyword arguments:
+
+            - **degree** : int, tuple, optional
+                Degree of the interpolating spline. A tuple can be used
+                to provide different degrees for the X- and Y-axes.
+                Default value is degree=3.
+
+            - **s** : float, optional
+                Non-negative smoothing factor. Default value s=0
+                corresponds to interpolation.  See
+                :py:class:`~scipy.interpolate.RectBivariateSpline` for
+                more details.
+
+        Notes
+        -----
+        * When subclassing :py:class:`FittableImageModel` for the
+          purpose of overriding :py:func:`compute_interpolator`, the
+          :py:func:`evaluate` may need to overriden as well depending on
+          the behavior of the new interpolator. In addition, for
+          improved future compatibility, make sure that the overriding
+          method stores keyword arguments ``kwargs`` by calling
+          ``_store_interpolator_kwargs`` method.
+
+        * Use caution when modifying interpolator's degree or smoothness
+          in a computationally intensive part of the code as it may
+          decrease code performance due to the need to recompute
+          interpolator.
+        """
+
+        from scipy.interpolate import RectBivariateSpline
+
+        if 'degree' in kwargs:
+            degree = kwargs['degree']
+            if hasattr(degree, '__iter__') and len(degree) == 2:
+                degx = int(degree[0])
+                degy = int(degree[1])
+            else:
+                degx = int(degree)
+                degy = int(degree)
+            if degx < 0 or degy < 0:
+                raise ValueError("Interpolator degree must be a non-negative "
+                                 "integer")
+        else:
+            degx = 3
+            degy = 3
+
+        if 's' in kwargs:
+            smoothness = kwargs['s']
+        else:
+            smoothness = 0
+
+        # Interpolator must be set to interpolate on the undersampled
+        # pixel grid, going from 0 to len(undersampled_grid)
+        x = np.arange(self._nx, dtype=np.float) / self.oversampling[0]
+        y = np.arange(self._ny, dtype=np.float) / self.oversampling[1]
+        self.interpolator = RectBivariateSpline(
+            x, y, self._data.T, kx=degx, ky=degy, s=smoothness)
+
+        self._store_interpolator_kwargs(**kwargs)
+
+    def evaluate(self, x, y, flux, x_0, y_0):
+        """
+        Evaluate the model on some input variables and provided model
+        parameters.
+        """
+
+        xi = np.asarray(x) - x_0 + self._x_origin
+        yi = np.asarray(y) - y_0 + self._y_origin
+
+        evaluated_model = flux * self.interpolator.ev(xi, yi)
+
+        if self._fill_value is not None:
+            # find indices of pixels that are outside the input pixel
+            # grid and set these pixels to the 'fill_value':
+            invalid = (((xi < 0) | (xi > (self._nx - 1)
+                                    / self.oversampling[0])) |
+                       ((yi < 0) | (yi > (self._ny - 1)
+                                    / self.oversampling[1])))
+            evaluated_model[invalid] = self._fill_value
+
+        return evaluated_model
 
 
 class GriddedPSFModel(Fittable2DModel):
@@ -733,7 +931,7 @@ class GriddedPSFModel(Fittable2DModel):
 
         xyref = [tuple(i) for i in xyref]
         idx = sorted(range(len(xyref)), key=xyref.__getitem__)
-        xyref = sorted(xyref)   # sort by x, then y
+        xyref = sorted(xyref)  # sort by x, then y
         (x0, y0), (_x0, y1), (x1, _y0), (_x1, _y1) = xyref
 
         if x0 != _x0 or x1 != _x1 or y0 != _y0 or y1 != _y1:
@@ -794,9 +992,11 @@ class GriddedPSFModel(Fittable2DModel):
 
 class IntegratedGaussianPRF(Fittable2DModel):
     r"""
-    Circular Gaussian model integrated over pixels. Because it is
-    integrated, this model is considered a PRF, *not* a PSF (see
-    :ref:`psf-terminology` for more about the terminology used here.)
+    Circular Gaussian model integrated over pixels.
+
+    Because it is integrated, this model is considered a PRF, *not* a
+    PSF (see :ref:`psf-terminology` for more about the terminology used
+    here.)
 
     This model is a Gaussian *integrated* over an area of ``1`` (in
     units of the model input coordinates, e.g. 1 pixel).  This is in
@@ -810,11 +1010,11 @@ class IntegratedGaussianPRF(Fittable2DModel):
     ----------
     sigma : float
         Width of the Gaussian PSF.
-    flux : float (default 1)
+    flux : float, optional
         Total integrated flux over the entire PSF
-    x_0 : float (default 0)
+    x_0 : float, optional
         Position of the peak in x direction.
-    y_0 : float (default 0)
+    y_0 : float, optional
         Position of the peak in y direction.
 
     Notes
@@ -878,8 +1078,9 @@ class IntegratedGaussianPRF(Fittable2DModel):
 
 class PRFAdapter(Fittable2DModel):
     """
-    A model that adapts a supplied PSF model to act as a PRF. It
-    integrates the PSF model over pixel "boxes".  A critical built-in
+    A model that adapts a supplied PSF model to act as a PRF.
+
+    It integrates the PSF model over pixel "boxes".  A critical built-in
     assumption is that the PSF model scale and location parameters are
     in *pixel* units.
 
