@@ -1,22 +1,24 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-"""Module which provides classes to perform PSF Photometry"""
+"""
+This module provides classes to perform PSF-fitting photometry.
+"""
 
-import numpy as np
 import warnings
 
 from astropy.modeling.fitting import LevMarLSQFitter
 from astropy.nddata.utils import overlap_slices
-from astropy.stats import gaussian_sigma_to_fwhm, SigmaClip
-from astropy.table import Table, Column, vstack, hstack
+from astropy.stats import SigmaClip, gaussian_sigma_to_fwhm
+from astropy.table import Column, Table, hstack, vstack
 from astropy.utils.exceptions import AstropyUserWarning
+import numpy as np
 
 from .groupstars import DAOGroup
-from .utils import (get_grouped_psf_model, subtract_psf,
-                    _extract_psf_fitting_names)
+from .utils import (_extract_psf_fitting_names, get_grouped_psf_model,
+                    subtract_psf)
 from ..aperture import CircularAperture, aperture_photometry
 from ..background import MMMBackground
 from ..detection import DAOStarFinder
-
+from ..utils.exceptions import NoDetectionsWarning
 
 __all__ = ['BasicPSFPhotometry', 'IterativelySubtractedPSFPhotometry',
            'DAOPhotPSFPhotometry']
@@ -48,7 +50,7 @@ class BasicPSFPhotometry:
         ``y_0``, and ``group_id``. The column ``group_id`` should cotain
         integers starting from ``1`` that indicate which group a given
         source belongs to. See, e.g., `~photutils.psf.DAOGroup`.
-    bkg_estimator : callable, instance of any `~photutils.BackgroundBase` subclass, or None
+    bkg_estimator : callable, instance of any `~photutils.background.BackgroundBase` subclass, or None
         ``bkg_estimator`` should be able to compute either a scalar
         background or a 2D background of a given 2D image. See, e.g.,
         `~photutils.background.MedianBackground`.  If None, no
@@ -86,10 +88,14 @@ class BasicPSFPhotometry:
         Fitter object used to compute the optimized centroid positions
         and/or flux of the identified sources. See
         `~astropy.modeling.fitting` for more details on fitters.
-    aperture_radius : float or None
+    aperture_radius : `None` or float
         The radius (in units of pixels) used to compute initial
-        estimates for the fluxes of sources. If ``None``, one FWHM will
-        be used if it can be determined from the ``psf_model``.
+        estimates for the fluxes of sources.  ``aperture_radius`` must
+        be set if initial flux guesses are not input to the photometry
+        class via the ``init_guesses`` keyword.  For tabular PSF models
+        (e.g. an `EPSFModel`), you must input the ``aperture_radius``
+        keyword.  For analytical PSF models, alternatively you may
+        define a FWHM attribute on your input psf_model.
 
     Notes
     -----
@@ -162,9 +168,7 @@ class BasicPSFPhotometry:
         elif value is None:
             self._aperture_radius = value
         else:
-            raise ValueError('aperture_radius must be a real-valued '
-                             'number, received aperture_radius = {}'
-                             .format(value))
+            raise ValueError('aperture_radius must be a positive number')
 
     def get_residual_image(self):
         """
@@ -239,16 +243,33 @@ class BasicPSFPhotometry:
                 self.aperture_radius = (self.psf_model.sigma.value *
                                         gaussian_sigma_to_fwhm)
 
+        if self.aperture_radius is None:
+            if init_guesses is None:
+                raise ValueError('aperture_radius was not input and could '
+                                 'not be determined by the input psf_model '
+                                 '(e.g. an EPSFModel).  For tabular PSF '
+                                 'models, you must input the aperture_radius '
+                                 'keyword.  For analytical PSF models, you '
+                                 'must either input the aperture_radius '
+                                 'keyword or define a fwhm or sigma '
+                                 'attribute on your input psf_model.')
+
+            if (init_guesses is not None and
+                    'flux_0' not in init_guesses.colnames):
+                raise ValueError('init_guesses were input, but the "flux_0" '
+                                 'column was not present.  Initial fluxes '
+                                 'cannot be calculated because '
+                                 'aperture_radius must was not input and '
+                                 'could not be determined by the input '
+                                 'psf_model (e.g. an EPSFModel).  For '
+                                 'analytical PSF models, you must either '
+                                 'input the aperture_radius keyword or '
+                                 'define a fwhm or sigma attribute on your '
+                                 'input psf_model.')
+
         if init_guesses is not None:
             # make sure the code does not modify user's input
             init_guesses = init_guesses.copy()
-            if self.aperture_radius is None:
-                if 'flux_0' not in init_guesses.colnames:
-                    raise ValueError('aperture_radius is None and could not '
-                                     'be determined by psf_model. Please, '
-                                     'either provided a value for '
-                                     'aperture_radius or define fwhm/sigma '
-                                     'at psf_model.')
 
             if self.finder is not None:
                 warnings.warn('Both init_guesses and finder are different '
@@ -256,8 +277,9 @@ class BasicPSFPhotometry:
                               'going to be ignored.', AstropyUserWarning)
 
             if 'flux_0' not in init_guesses.colnames:
-                apertures = CircularAperture((init_guesses['x_0'],
-                                              init_guesses['y_0']),
+                positions = np.transpose((init_guesses['x_0'],
+                                          init_guesses['y_0']))
+                apertures = CircularAperture(positions,
                                              r=self.aperture_radius)
 
                 init_guesses['flux_0'] = aperture_photometry(
@@ -268,8 +290,9 @@ class BasicPSFPhotometry:
                                  'not given.')
             sources = self.finder(image)
             if len(sources) > 0:
-                apertures = CircularAperture((sources['xcentroid'],
-                                              sources['ycentroid']),
+                positions = np.transpose((sources['xcentroid'],
+                                          sources['ycentroid']))
+                apertures = CircularAperture(positions,
                                              r=self.aperture_radius)
 
                 sources['aperture_flux'] = aperture_photometry(
@@ -338,7 +361,7 @@ class BasicPSFPhotometry:
             group_psf = get_grouped_psf_model(self.psf_model,
                                               star_groups.groups[n],
                                               self._pars_to_set)
-            usepixel = np.zeros_like(image, dtype=np.bool)
+            usepixel = np.zeros_like(image, dtype=bool)
 
             for row in star_groups.groups[n]:
                 usepixel[overlap_slices(large_array_shape=image.shape,
@@ -493,7 +516,7 @@ class IterativelySubtractedPSFPhotometry(BasicPSFPhotometry):
         ``y_0``, and ``group_id``. The column ``group_id`` should cotain
         integers starting from ``1`` that indicate which group a given
         source belongs to. See, e.g., `~photutils.psf.DAOGroup`.
-    bkg_estimator : callable, instance of any `~photutils.BackgroundBase` subclass, or None
+    bkg_estimator : callable, instance of any `~photutils.background.BackgroundBase` subclass, or None
         ``bkg_estimator`` should be able to compute either a scalar
         background or a 2D background of a given 2D image. See, e.g.,
         `~photutils.background.MedianBackground`.  If None, no
@@ -651,6 +674,8 @@ class IterativelySubtractedPSFPhotometry(BasicPSFPhotometry):
         else:
             if self.bkg_estimator is not None:
                 self._residual_image = image - self.bkg_estimator(image)
+            else:
+                self._residual_image = image
 
             if self.aperture_radius is None:
                 if hasattr(self.psf_model, 'fwhm'):
@@ -696,10 +721,11 @@ class IterativelySubtractedPSFPhotometry(BasicPSFPhotometry):
         sources = self.finder(self._residual_image)
 
         n = n_start
-        while(len(sources) > 0 and
+        while((sources is not None and len(sources) > 0) and
               (self.niters is None or n <= self.niters)):
-            apertures = CircularAperture((sources['xcentroid'],
-                                          sources['ycentroid']),
+            positions = np.transpose((sources['xcentroid'],
+                                      sources['ycentroid']))
+            apertures = CircularAperture(positions,
                                          r=self.aperture_radius)
             sources['aperture_flux'] = aperture_photometry(
                 self._residual_image, apertures)['aperture_sum']
@@ -731,7 +757,7 @@ class IterativelySubtractedPSFPhotometry(BasicPSFPhotometry):
 
             # do not warn if no sources are found beyond the first iteration
             with warnings.catch_warnings():
-                warnings.simplefilter('ignore', AstropyUserWarning)
+                warnings.simplefilter('ignore', NoDetectionsWarning)
                 sources = self.finder(self._residual_image)
 
             n += 1
