@@ -101,8 +101,8 @@ class Segment:
     @lazyproperty
     def bbox(self):
         """
-        The `~photutils.BoundingBox` of the minimal rectangular region
-        containing the source segment.
+        The `~photutils.aperture.BoundingBox` of the minimal rectangular
+        region containing the source segment.
         """
 
         if self._segment_data.ndim != 2:
@@ -766,7 +766,7 @@ class SegmentationImage:
 
         if relabel:
             labels = np.unique(idx[idx != 0])
-            idx2 = np.zeros(max(labels) + 1, dtype=np.int)
+            idx2 = np.zeros(max(labels) + 1, dtype=int)
             idx2[labels] = np.arange(len(labels)) + 1
             idx = idx2[idx]
 
@@ -811,7 +811,7 @@ class SegmentationImage:
                 (self.labels[-1] - self.labels[0] + 1) == self.nlabels):
             return
 
-        new_labels = np.zeros(self.max_label + 1, dtype=np.int)
+        new_labels = np.zeros(self.max_label + 1, dtype=int)
         new_labels[self.labels] = np.arange(self.nlabels) + start_label
 
         data_new = new_labels[self.data]
