@@ -70,14 +70,14 @@ class SourceProperties:
         which to calculate the source centroid and morphological
         properties.  The kernel used to perform the filtering should be
         the same one used in defining the source segments (e.g., see
-        :func:`~photutils.detect_sources`).  If ``data`` is a
-        `~astropy.units.Quantity` array then ``filtered_data`` must be a
-        `~astropy.units.Quantity` array (and vise versa) with identical
-        units.  Non-finite ``filtered_data`` values (NaN and +/- inf)
-        are not automatically masked, unless they are at the same
-        position of non-finite values in the input ``data`` array.  Such
-        pixels can be masked using the ``mask`` keyword.  If `None`,
-        then the unfiltered ``data`` will be used instead.
+        :func:`~photutils.segmentation.detect_sources`).  If ``data`` is
+        a `~astropy.units.Quantity` array then ``filtered_data`` must be
+        a `~astropy.units.Quantity` array (and vise versa) with
+        identical units.  Non-finite ``filtered_data`` values (NaN and
+        +/- inf) are not automatically masked, unless they are at the
+        same position of non-finite values in the input ``data`` array.
+        Such pixels can be masked using the ``mask`` keyword.  If
+        `None`, then the unfiltered ``data`` will be used instead.
 
     error : array_like or `~astropy.units.Quantity`, optional
         The total error array corresponding to the input ``data`` array.
@@ -142,32 +142,33 @@ class SourceProperties:
     properties based on image moments.  Negative values could occur, for
     example, if the segmentation image was defined from a different
     image (e.g., different bandpass) or if the background was
-    oversubtracted. Note that `~photutils.SourceProperties.source_sum`
-    always includes the contribution of negative ``data`` values.
+    oversubtracted. Note that
+    `~photutils.segmentation.SourceProperties.source_sum` always
+    includes the contribution of negative ``data`` values.
 
     The input ``error`` array is assumed to include *all* sources of
     error, including the Poisson error of the sources.
-    `~photutils.SourceProperties.source_sum_err` is simply the
-    quadrature sum of the pixel-wise total errors over the non-masked
-    pixels within the source segment:
+    `~photutils.segmentation.SourceProperties.source_sum_err` is simply
+    the quadrature sum of the pixel-wise total errors over the
+    non-masked pixels within the source segment:
 
     .. math:: \\Delta F = \\sqrt{\\sum_{i \\in S}
               \\sigma_{\\mathrm{tot}, i}^2}
 
     where :math:`\\Delta F` is
-    `~photutils.SourceProperties.source_sum_err`, :math:`S` are the
-    non-masked pixels in the source segment, and
+    `~photutils.segmentation.SourceProperties.source_sum_err`, :math:`S`
+    are the non-masked pixels in the source segment, and
     :math:`\\sigma_{\\mathrm{tot}, i}` is the input ``error`` array.
 
     Custom errors for source segments can be calculated using the
-    `~photutils.SourceProperties.error_cutout_ma` and
-    `~photutils.SourceProperties.background_cutout_ma` properties, which
-    are 2D `~numpy.ma.MaskedArray` cutout versions of the input
-    ``error`` and ``background``.  The mask is `True` for pixels outside
-    of the source segment, masked pixels from the ``mask`` input, or any
-    non-finite ``data`` values (NaN and +/- inf).
+    `~photutils.segmentation.SourceProperties.error_cutout_ma` and
+    `~photutils.segmentation.SourceProperties.background_cutout_ma`
+    properties, which are 2D `~numpy.ma.MaskedArray` cutout versions of
+    the input ``error`` and ``background``.  The mask is `True` for
+    pixels outside of the source segment, masked pixels from the
+    ``mask`` input, or any non-finite ``data`` values (NaN and +/- inf).
 
-    .. _SExtractor: http://www.astromatic.net/software/sextractor
+    .. _SExtractor: https://www.astromatic.net/software/sextractor
     """
 
     def __init__(self, data, segment_img, label, filtered_data=None,
@@ -334,11 +335,15 @@ class SourceProperties:
         etc.
         """
 
+        if isinstance(self.data_cutout, u.Quantity):
+            cutout = self.data_cutout.value
+        else:
+            cutout = self.data_cutout
+
         # NOTE: using np.where is faster than
         #     _data = np.copy(self.data_cutout)
         #     self._data[self._total_mask] = 0.
-        return np.where(self._total_mask, 0,
-                        self.data_cutout).astype(np.float64)  # copy
+        return np.where(self._total_mask, 0, cutout).astype(float)  # copy
 
     @lazyproperty
     def _filtered_data_zeroed(self):
@@ -358,9 +363,12 @@ class SourceProperties:
         """
 
         filt_data = self._filtered_data[self.slices]
+        if isinstance(filt_data, u.Quantity):
+            filt_data = filt_data.value
+
         filt_data = np.where(self._total_mask, 0., filt_data)  # copy
         filt_data[filt_data < 0] = 0.
-        return filt_data.astype(np.float64)
+        return filt_data.astype(float)
 
     def make_cutout(self, data, masked_array=False):
         """
@@ -692,8 +700,8 @@ class SourceProperties:
     @lazyproperty
     def bbox(self):
         """
-        The `~photutils.BoundingBox` of the minimal rectangular region
-        containing the source segment.
+        The `~photutils.aperture.BoundingBox` of the minimal rectangular
+        region containing the source segment.
         """
 
         return BoundingBox(self.slices[1].start, self.slices[1].stop,
@@ -1002,7 +1010,8 @@ class SourceProperties:
     @lazyproperty
     def source_sum_err(self):
         """
-        The uncertainty of `~photutils.SourceProperties.source_sum`,
+        The uncertainty of
+        `~photutils.segmentation.SourceProperties.source_sum`,
         propagated from the input ``error`` array.
 
         ``source_sum_err`` is the quadrature sum of the total errors
@@ -1145,7 +1154,7 @@ class SourceProperties:
             data = ~self._total_mask
             selem = np.array([[0, 1, 0], [1, 1, 1], [0, 1, 0]])
             data_eroded = binary_erosion(data, selem, border_value=0)
-            border = np.logical_xor(data, data_eroded).astype(np.int)
+            border = np.logical_xor(data, data_eroded).astype(int)
 
             kernel = np.array([[10, 2, 10], [2, 1, 2], [10, 2, 10]])
             perimeter_data = convolve(border, kernel, mode='constant', cval=0)
@@ -1154,7 +1163,7 @@ class SourceProperties:
             perimeter_hist = np.bincount(perimeter_data.ravel(),
                                          minlength=size)
 
-            weights = np.zeros(size, dtype=np.float)
+            weights = np.zeros(size, dtype=float)
             weights[[5, 7, 15, 17, 25, 27]] = 1.
             weights[[21, 33]] = np.sqrt(2.)
             weights[[13, 23]] = (1 + np.sqrt(2.)) / 2.
@@ -1219,13 +1228,14 @@ class SourceProperties:
         order.
         """
 
-        if np.any(~np.isfinite(self.covariance)):
-            return (np.nan, np.nan) * u.pix**2
+        unit = u.pix**2  # eigvals unit
+        if np.any(~np.isfinite(self.covariance.value)):
+            return (np.nan, np.nan) * unit
         else:
-            eigvals = np.linalg.eigvals(self.covariance)
+            eigvals = np.linalg.eigvals(self.covariance.value)
             if np.any(eigvals < 0):  # negative variance
-                return (np.nan, np.nan) * u.pix**2  # pragma: no cover
-            return (np.max(eigvals), np.min(eigvals)) * u.pix**2
+                return (np.nan, np.nan) * unit  # pragma: no cover
+            return (np.max(eigvals), np.min(eigvals)) * unit
 
     @lazyproperty
     def semimajor_axis_sigma(self):
@@ -1508,8 +1518,8 @@ def source_properties(data, segment_img, error=None, mask=None,
         calculating the source centroid and morphological parameters.
         The kernel should be the same one used in defining the source
         segments, i.e. the detection image (e.g., see
-        :func:`~photutils.detect_sources`).  If `None`, then the
-        unfiltered ``data`` will be used instead.
+        :func:`~photutils.segmentation.detect_sources`).  If `None`,
+        then the unfiltered ``data`` will be used instead.
 
     wcs : `None` or WCS object, optional
         A world coordinate system (WCS) transformation that supports the
@@ -1546,24 +1556,25 @@ def source_properties(data, segment_img, error=None, mask=None,
     properties based on image moments.  Negative values could occur, for
     example, if the segmentation image was defined from a different
     image (e.g., different bandpass) or if the background was
-    oversubtracted. Note that `~photutils.SourceProperties.source_sum`
-    always includes the contribution of negative ``data`` values.
+    oversubtracted. Note that
+    `~photutils.segmentation.SourceProperties.source_sum` always
+    includes the contribution of negative ``data`` values.
 
     The input ``error`` is assumed to include *all* sources of error,
     including the Poisson error of the sources.
-    `~photutils.SourceProperties.source_sum_err` is simply the
-    quadrature sum of the pixel-wise total errors over the non-masked
-    pixels within the source segment:
+    `~photutils.segmentation.SourceProperties.source_sum_err` is simply
+    the quadrature sum of the pixel-wise total errors over the
+    non-masked pixels within the source segment:
 
     .. math:: \\Delta F = \\sqrt{\\sum_{i \\in S}
               \\sigma_{\\mathrm{tot}, i}^2}
 
     where :math:`\\Delta F` is
-    `~photutils.SourceProperties.source_sum_err`, :math:`S` are the
-    non-masked pixels in the source segment, and
+    `~photutils.segmentation.SourceProperties.source_sum_err`, :math:`S`
+    are the non-masked pixels in the source segment, and
     :math:`\\sigma_{\\mathrm{tot}, i}` is the input ``error`` array.
 
-    .. _SExtractor: http://www.astromatic.net/software/sextractor
+    .. _SExtractor: https://www.astromatic.net/software/sextractor
 
     See Also
     --------
@@ -1788,10 +1799,10 @@ class SourceCatalog:
         scalar-valued properties.
 
         Multi-dimensional properties, e.g.
-        `~photutils.SourceProperties.data_cutout`, can be included in
-        the ``columns`` input, but they will not be preserved when
-        writing the table to a file.  This is a limitation of
-        multi-dimensional columns in astropy tables.
+        `~photutils.segmentation.SourceProperties.data_cutout`, can be
+        included in the ``columns`` input, but they will not be
+        preserved when writing the table to a file.  This is a
+        limitation of multi-dimensional columns in astropy tables.
 
         Parameters
         ----------
@@ -1916,7 +1927,7 @@ def _calc_sky_bbox_corner(bbox, corner, wcs):
 
     Parameters
     ----------
-    bbox : `~photutils.BoundingBox`
+    bbox : `~photutils.aperture.BoundingBox`
         The source bounding box.
 
     corner : {'ll', 'ul', 'lr', 'ur'}
