@@ -1,8 +1,11 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
+"""
+This module provides tools to calculate statistics.
+"""
 
+from astropy.utils import deprecated
 import numpy as np
 
-
 __all__ = ['std_blocksum']
 
 
@@ -50,6 +53,7 @@ def _mesh_values(data, box_size):
     return data[idx]
 
 
+@deprecated('0.7')
 def std_blocksum(data, block_sizes, mask=None):
     """
     Calculate the standard deviation of block-summed data values at
@@ -79,6 +83,38 @@ def std_blocksum(data, block_sizes, mask=None):
         for the input ``block_sizes``.
     """
 
+    return _std_blocksum(data, block_sizes, mask=mask)
+
+
+def _std_blocksum(data, block_sizes, mask=None):
+    """
+    Calculate the standard deviation of block-summed data values at
+    sizes of ``block_sizes``.
+
+    Values from incomplete blocks, either because of the image edges or
+    masked pixels, are not included.
+
+    Parameters
+    ----------
+    data : array-like
+        The 2D array to block sum.
+
+    block_sizes : int, array-like of int
+        An array of integer (square) block sizes.
+
+    mask : array-like (bool), optional
+        A boolean mask, with the same shape as ``data``, where a `True`
+        value indicates the corresponding element of ``data`` is masked.
+        Blocks that contain *any* masked data are excluded from
+        calculations.
+
+    Returns
+    -------
+    result : `~numpy.ndarray`
+        An array of the standard deviations of the block-summed array
+        for the input ``block_sizes``.
+    """
+
     data = np.ma.asanyarray(data)
 
     if mask is not None and mask is not np.ma.nomask:
